/**
 * Computes the deblending residual
 * after signal and noise separation
 * @author: Joseph Jennings
 * @version: 2017.03.29
 */

#include "sep_params.h"
#include "../main/simpleplot.h"

int main(int argc, char **argv) {

  sep_params pars = sep_params(argc,argv);

  sregf datin = sregf("in");
  sregf smvin = sregf("smov");
  sregf nmvin = sregf("nmov");

  int nt = datin.get_axis(1).n;
  int nr = datin.get_axis(2).n;
  int niter = smvin.get_axis(3).n;
  std::vector<axis> nrmxs;
  nrmxs.push_back(axis(niter,0.0,1.0));

  hypercube_float *dat  = new hypercube_float(datin.get_axes(),true);
  hypercube_float *rsd  = new hypercube_float(datin.get_axes(),true);
  hypercube_float *tsig = new hypercube_float(datin.get_axes(),true);
  hypercube_float *tnoz = new hypercube_float(datin.get_axes(),true);
  hypercube_float *sig  = new hypercube_float(smvin.get_axes(),true);
  hypercube_float *noz  = new hypercube_float(nmvin.get_axes(),true);
  hypercube_float *nrm  = new hypercube_float(nrmxs,true);

  datin.read_all("in",dat);
  smvin.read_all("smov",sig);
  nmvin.read_all("nmov",noz);

  for(int iter = 0; iter < niter; ++iter) {
    memcpy(tsig->vals,&sig->vals[(iter)*nt*nr],sizeof(float)*nt*nr);
    memcpy(tnoz->vals,&noz->vals[(iter)*nt*nr],sizeof(float)*nt*nr);
    rsd->scale_add(1.0,tsig,1.0,tnoz);
    rsd->scale_add(1.0,dat,-1.0);
    nrm->vals[iter] = rsd->dot(rsd);
  }

  sregf nout = sregf(nrm->get_axes());
  int one = 1;
  nout.write_description("out");
  putch("n2","d",&one);
  putch("n3","d",&one);
  nout.write_all("out",nrm);

  delete dat; delete rsd; delete tsig; delete tnoz;
  delete sig; delete noz; delete nrm;
}
