/**
 * Generate an ellipse from
 * phase-shifted sinusoids
 * @author: Joseph Jennings
 * @version: 2017.02.17
 */

#include "sep_params.h"
#include "wavelet.h"
#include "hypercube_complex.h"
#include "sregc.h"

int main(int argc, char **argv) {

  sep_params pars = sep_params(argc,argv);
  int nt = pars.get_int("nt",256);
  float dt = pars.get_float("dt",0.001);
  float ampy = pars.get_float("ampy",1.0);
  float ampx = pars.get_float("ampx",1.0);
  float phz = pars.get_float("phz",90.0);
  float frq = pars.get_float("frq",4.0);

  std::vector<axis> at;
  at.push_back(axis(nt,0.0,dt));

  hypercube_float *x = new hypercube_float(at,true);
  hypercube_float *y = new hypercube_float(at,true);

  wavelet wav(at[0]);

  wav.monosin(frq,ampx,phz,x);
  wav.monosin(frq,ampy,0.0,y);

  hypercube_complex *ell = new hypercube_complex(at,true);

  for(int it = 0; it < nt; ++it) {
    ell->vals[it] = std::complex<float>(x->vals[it],y->vals[it]);
  }

  sregc fout(ell->get_axes());
  fout.write_description("out");
  fout.write_all("out",ell);

  delete ell; delete x; delete y;
}
