/**
 * Tests the wavefield integration
 * operator forward.
 * @author: Joseph Jennings
 * @version: 2017.02.14
 */

#include "sep_params.h"
#include "zeropad.h"
#include "hypercube_float.h"
#include "../main/integratewfld.h"

int main(int argc, char **argv) {

  sep_params pars = sep_params(argc,argv);
  int nx = pars.get_int("nx",300);
  int nz = pars.get_int("nz",300);
  float rho = pars.get_float("rho",1000.0);

  sregf fin = sregf("in");

  std::vector<axis> waxes;
  waxes.push_back(axis(nz,0.0,1.0));
  waxes.push_back(axis(nx,0.0,1.0));
  waxes.push_back(fin.get_axis(1));

  hypercube_float *tsig = new hypercube_float(fin.get_axes(),true);
  hypercube_float *wfld = new hypercube_float(waxes,true);
  hypercube_float *intr = new hypercube_float(waxes,true);
  hypercube_float *tint = new hypercube_float(fin.get_axes(),true);
  fin.read_all("in",tsig);

  zeropad zp = zeropad(100,100);
  zp.forward(false,tsig,wfld);

  integratewfld iw = integratewfld(waxes,rho);

  iw.forward(false,wfld,intr);

  zp.adjoint(false,tint,intr);

  sregf fout = sregf(tint->get_axes());
  fout.write_description("out");
  fout.write_all("out",tint);

  delete tsig; delete tint;
  delete wfld; delete intr;
}
