/**
 * Tests the adjoint of the multicomponent
 * receiver restriction operator
 * @author: Joseph Jennings
 * @version: 2017.04.09
 */

#include <limits.h>
#include "gtest/gtest.h"
#include "../main/mcrrec.h"
#include "axis.h"
#include "hypercube_float.h"

class mcrrectest: public ::testing::Test {
  protected:
    virtual void SetUp(){
    }

    virtual void TearDown() {
    }
};

TEST_F(mcrrectest,dptest){
  int nx = 300; int nz = 300;
  int nt = 250; int nrx = 100;
  float rho = 1000.0;
  /* Build axes */
  axis az = axis(nz, 0.0, 1.0);
  axis ax = axis(nx, 0.0, 1.0);
  axis at = axis(nt, 0.0, 1.0);
  axis ar = axis(nrx,0.0, 1.0);
  axis ac = axis(3,  0.0 ,1.0);
  std::vector<axis> maxes, daxes;
  daxes.push_back(az); daxes.push_back(ax);
  daxes.push_back(at);
  maxes.push_back(at); maxes.push_back(ar);
  maxes.push_back(ac);

  hypercube_float *m  = new hypercube_float(maxes,true);
  hypercube_float *ms = new hypercube_float(maxes,true);
  hypercube_float *d  = new hypercube_float(daxes,true);
  hypercube_float *ds = new hypercube_float(daxes,true);

  mcrrec mcrr = mcrrec(daxes,ar,70,rho);

  m->random(); d->random();

  /* Add == false */
  mcrr.forward(false,m,ds);
  double dotd = d->dot(ds);
  mcrr.adjoint(false,ms,d);
  double dotm = m->dot(ms);
  fprintf(stderr,"%f %f\n",dotd,dotm);
  ASSERT_NEAR(dotd,dotm,0.00001);

  delete m; delete d; delete ms; delete ds;
}
