/**
 * Tests forward and adjoint of forward
 * linear prediction and mask chain operator
 * @author: Joseph Jennings
 * @version: 2017.03.05
 */

#include <limits.h>
#include <random>
#include "gtest/gtest.h"
#include "../main/prediction.h"
#include "../main/pefmask.h"
#include "combo_oper.h"
#include "axis.h"
#include "hypercube_float.h"

class pefmasktest: public ::testing::Test {
  protected:
    virtual void SetUp(){
    }

    virtual void TearDown() {
    }
};

TEST_F(pefmasktest,dptest) {
  int nd = 100; int nm = 20;
  std::vector<axis> daxes, maxes;
  daxes.push_back(axis(nd,0.0,1.0));
  maxes.push_back(axis(nm,0.0,1.0));

  hypercube_float *datop = new hypercube_float(daxes,true);
  hypercube_float *m  = new hypercube_float(maxes,true);
  hypercube_float *ms = new hypercube_float(maxes,true);
  hypercube_float *d  = new hypercube_float(daxes,true);
  hypercube_float *ds = new hypercube_float(daxes,true);

  datop->random(); m->random(); d->random();

  prediction *pd = new prediction(datop,maxes[0],true);
  pefmask *k = new pefmask();

  pd->set_domain(m); pd->set_range(d);
  k->set_domain(m); k->set_range(m);
  chain_op *dk = new chain_op(k,pd);

  dk->forward(false,m,ds);
  float dotd = d->dot(ds);
  dk->adjoint(false,ms,d);
  float dotm = m->dot(ms);
  fprintf(stderr,"dotm=%f dotd=%f\n",dotm,dotd);
  ASSERT_NEAR(dotm,dotd,0.000001);

  delete m; delete datop; delete ms; delete d; delete ds;
  delete pd; delete k; delete dk;
}

TEST_F(pefmasktest, restest) {
  std::vector<float> dop = {1,1,1,1,1,1,1};

  int nd = dop.size(); int nf = 2;
  std::vector<axis> daxes,faxes;
  daxes.push_back(axis(nd,0.0,1.0));
  faxes.push_back(axis(nf,0.0,1.0));

  hypercube_float *datop = new hypercube_float(daxes,true);
  hypercube_float *m     = new hypercube_float(faxes,true);
  hypercube_float *d     = new hypercube_float(daxes,true);
  hypercube_float *t     = new hypercube_float(daxes,true);

  datop->vals = dop.data();
  /* Initialize filter coefficients */
  m->vals[0] = 1.0; m->vals[1] = -1.0;

  prediction *pd = new prediction(datop,faxes[0],true);
  pefmask *k = new pefmask();

  pd->set_domain(m); pd->set_range(d);
  k->set_domain(m); k->set_range(m);
  chain_op *dk = new chain_op(k,pd);

  pd->setdata(d);
  d->scale(-1.0);

  /* Apply pef + mask */
  dk->forward(false,m,t);

  /* Add vector */
  t->scale_add(1.0,d,-1.0);

  ASSERT_EQ(t->sum(),0.00000);

}
