/**
 * A main program to test the shot
 * pad operator. Pads a single shot in time.
 * @author: Joseph Jennings
 * @version: 2017.01.22
 */

#include <vector>
#include "sep_params.h"
#include "hypercube_float.h"
#include "../main/shotpad.h"

int main(int argc, char **argv) {

  sep_params pars = sep_params(argc, argv);
  int adj = pars.get_int("adj",0);
  int ntp = pars.get_int("ntp",600);
  int ntc = pars.get_int("ntc",200);
  float shift = pars.get_float("shift",0.2);

  sregf in = sregf("in");

  std::vector<axis> paxes, daxes;
  paxes.push_back(axis(ntp,in.get_axis(1).o,in.get_axis(1).d));
  paxes.push_back(in.get_axis(2));
  daxes.push_back(axis(ntc,in.get_axis(1).o,in.get_axis(1).d));
  daxes.push_back(in.get_axis(2));

  hypercube_float *dat = nullptr; hypercube_float *pad = nullptr;

  if(adj) {
    /* Create operator */
    shotpad sp = shotpad(daxes[0]);
    sp.set_time(shift);
    /* Build hypercubes */
    pad = new hypercube_float(in.get_axes(),true);
    in.read_all("in",pad);
    dat = new hypercube_float(daxes,true);
    /* Apply operator */
    sp.adjoint(false,dat,pad);
    /* Write out data */
    sregf datout = sregf(dat->get_axes());
    datout.write_description("out");
    datout.write_all("out",dat);
  } else {
    /* Create operator */
    shotpad sp = shotpad(in.get_axis(1));
    sp.set_time(shift);
    /* Build hypercubes */
    dat = new hypercube_float(in.get_axes(),true);
    in.read_all("in",dat);
    pad = new hypercube_float(paxes,true);
    /* Apply operator */
    sp.forward(false,dat,pad);
    /* Write out data */
    sregf padout = sregf(pad->get_axes());
    padout.write_description("out");
    padout.write_all("out",pad);
  }

  delete dat; delete pad;

}
