#include "lin_solver.h"
#include "i_op.h"
#include "super_vector.h"
#include "combo_oper.h"
#include "timer.h"

void lin_solver::set_solver_defaults(){
  wt=0;
  m0=0;
  mask_op=0;
}
norm_vector *lin_solver::init_rr(){
  norm_vector *rtemp;
 if(wt!=0){
   if(!wt->domain->check_match(o_p->range)) seperr("wt and op don't match");
   rtemp=(norm_vector*)wt->range->clone_zero();
   wt->forward(false,d,rtemp);
   rtemp->scale(-1.);
   rtemp->set_norm(d->_norm->clone());
   rtemp->perc=d->perc;

   if(m0!=0){
     norm_vector *dtemp=(norm_vector*)d->clone_zero();

     o_p->forward(false,m0,dtemp);
     wt->forward(true,dtemp,rtemp);

     delete dtemp;
   }
  }
  else {
     rtemp=(norm_vector*)d->clone_vec();
     rtemp->scale(-1.);
     if(m0!=0) o_p->forward(true,m0,rtemp);
  }
 
  return rtemp;
}
void lin_solver::create_wt_op(){
  if(mask_op==0){
    if(wt!=0){
       wt_op=new chain_op(o_p,wt); have_wt=true;
    }
    else{
       wt_op=o_p; 
    }
  }
  else{
    if(wt!=0){
      wt_op=new chain_op(mask_op,o_p,wt);
      have_wt=true;
    }
    else wt_op=new chain_op(mask_op,o_p);
  }
}
void lin_solver::create_solver_vecs(){
   gg=(norm_vector*)rr->clone_zero();
   m=(norm_vector*) oper->domain->clone_zero();
   g=(norm_vector*) oper->domain->clone_zero();
   st->alloc_step(m,rr);

   rw=(norm_vector*)rr->clone_vec();
   if(m0!=0) m->scale_add(0.,m0,1.);
}
void lin_solver::init_solve(step *s){
  st=s;
  verb=0;
  have_wt=false;
  have_op=false;

}
void lin_solver::clean_up(){
   if(rr!=0)  delete rr; 
   if(rw!=0) delete rw;
   if(m!=0) delete m; 
   if(gg!=0) delete gg; 
   if(g!=0) delete g; 
   if(have_wt) delete wt_op;
   if(have_op) delete oper;
}
void lin_solver::init_lin_solver(){
   rr=0; rw=0; m=0; gg=0; g=0; wt_op=0; oper=0;
}
void lin_solver::set_verbose(int iverb){ verb=iverb;}
bool lin_solver::solve(int niter){
double scale;
  if(verb>0){
     scale=rr->dot(rr);
    fprintf(stderr,"Initial residual %g scaled to 10000\n",scale);
    scale=100000/scale;
  }


  for(int iter =0; iter < niter; iter++){
    rr->norm_scale(rw);
    if(verb>0) fprintf(stderr,"ITERATING %d %f \n",iter,rw->dot(rw)*scale);
    oper->adjoint(false,g,rw);
    oper->forward(false,g,gg);
    double val;
    bool valid=st->step_it(iter,m,g,rr,gg,&val);
    if(!valid){
       fprintf(stderr,"STEP RETURNED FALSE \n");
       return false;
    }
  }


  return true;
}
my_vector *lin_solver::return_model(){ 
 my_vector *x=m->clone_vec();
 return x;
}
void lin_solver::update_model(norm_vector *mod){mod->add(m);}
simple_solver::simple_solver(step *s, norm_vector *data, my_operator *op){
  
  set_solver_defaults();
  init_solve(s);
  o_p=op;
  d=data;
}
void simple_solver::setup_solver(){
  rr=init_rr();
  create_wt_op();
  oper=wt_op;
  create_solver_vecs();
  
}
void simple_solver::clean_up(){}
prec_solver::prec_solver(step *s, norm_vector *data, my_operator *op, my_operator *prec, 
float e){
  
   set_prec_defaults();
   init_solve(s);
   have_op=true;
   o_p=op;
   d=data;
   eps=e;
   pop=prec;
}
void prec_solver::setup_solver(){
  norm_vector *rtemp=init_rr();
  norm_vector *prec_domain;

  if(res_prec!=0) prec_domain=(norm_vector*)res_prec->clone_vec();
  else prec_domain=(norm_vector*)pop->domain->clone_zero();
  prec_domain->scale(eps);
  if(eps_op!=0) iop=eps_op;
  else iop=new i_op(prec_domain,prec_domain);
  iop->set_scale(eps);
  if(p0!=0) iop->forward(true,p0,prec_domain);

  create_wt_op();
  if(pmask_op!=0)prec_chain_op=new chain_op(pmask_op,pop,wt_op);
  else prec_chain_op=new chain_op(pop,wt_op);
  if(pmask_op!=0){
    mask_prec=new chain_op(pmask_op,iop);
    oper=new col_op(prec_chain_op,mask_prec);
  }
  else {
    oper=new col_op(prec_chain_op,iop);
  }
  rr=oper->range_vec(rtemp,prec_domain);

  delete prec_domain; delete rtemp;

  create_solver_vecs();

}
norm_vector *prec_solver::init_rr(){
  
  norm_vector *rtemp;
 if(wt!=0){
   if(!wt->domain->check_match(o_p->range)) seperr("wt and op don't match");
   rtemp=(norm_vector*)wt->range->clone_zero();
   wt->forward(false,d,rtemp);
   rtemp->scale(-1.);

   if(p0!=0){
      norm_vector *mt=(norm_vector*)pop->range->clone_vec();
      pop->forward(false,p0,mt);
      norm_vector *use=mt;
      if(mask_op!=0){
        norm_vector *mtt=(norm_vector*)mask_op->range->clone_vec();
        mask_op->forward(false,mt,mtt);
        delete mt;
        use=mtt;
      }
      norm_vector *dtemp=(norm_vector*)d->clone_zero();
      o_p->forward(false,use,dtemp);
      delete use;
      wt->forward(true,dtemp,rtemp);
      delete dtemp;
   }
  }
  else {
     rtemp=(norm_vector*)d->clone_vec();
     rtemp->scale(-1.);
     if(p0!=0){
       my_vector *opm=o_p->domain->clone_vec();
       if(mask_op!=0){
         my_vector *mt=mask_op->domain->clone_vec();
         pop->forward(false,p0,mt);
         mask_op->forward(false,mt,opm);
         o_p->forward(true,opm,rtemp);
         delete mt;
       }
       else{
         pop->forward(false,p0,opm);
         o_p->forward(false,opm,rtemp);
       }
       delete opm;
     }
  }
  return rtemp;
}
void prec_solver::set_prec_defaults(){
  set_solver_defaults();
  pmask_op=0;
  p0=0;
  res_prec=0;
  mask_prec=0;
  eps_op=0;
}
void prec_solver::clean_up(){
   if(eps_op==0) delete iop;
   delete prec_chain_op;
   if(mask_prec!=0) delete mask_prec;
}
my_vector *prec_solver::return_model(){
  my_vector *mod=pop->range->clone_zero();
  pop->forward(false,m,mod);
 

  return mod;
}
void prec_solver::update_model(norm_vector *mod){
  pop->forward(true,m,mod);
}
reg_solver::reg_solver(step *s, norm_vector *data, my_operator *op, my_operator *reg, 
  float e){
  
    set_reg_defaults();
    init_solve(s);
    o_p=op; eps=e; rop=reg; d=data;
    have_op=true;
}
void reg_solver::set_reg_defaults(){
  set_solver_defaults();
  res_reg=0;


}
void reg_solver::setup_solver(){
    
     rop_chain=0;

    norm_vector *rtemp=init_rr();
    norm_vector *reg_domain;
    if(res_reg!=0) reg_domain=(norm_vector*)res_reg->clone_vec();
    else reg_domain=(norm_vector*)rop->range->clone_zero();
    reg_domain->scale(eps);
    rop->set_scale(eps);

    if(m0!=0) {
      // m0->scale(-1.);
       rop->forward(true,m0,reg_domain);
     //  m0->scale(-1.);
    }
    create_wt_op();

    if(mask_op!=0){
       rop_chain=new chain_op(mask_op,rop);
       oper=new col_op(wt_op,rop_chain);
    }
    else  oper=new col_op(wt_op,rop);

//  rtemp->info("v1");
 // reg_domain->info("v3");
//fprintf(stderr,"bnlah %s %s \n",rtemp->_norm->what,reg_domain->_norm->what);
    rr=oper->range_vec(rtemp,reg_domain);
    create_solver_vecs();

}
void reg_solver::clean_up(){}




