#ifndef BASIC_SOLVER_H
#include"norm_vector.h"
#include"my_operator.h"
#include"cgstep.h"
#include <vector>
#include <assert.h>

class lin_solver{
  public:
    lin_solver(){init_lin_solver();}
    
    
    
    ~lin_solver(){clean_up();}
    void set_verbose(int iv);
    bool solve(int niter);
    void init_solve(step *s);
    void set_mask_op(my_operator *m){mask_op=m;}
    void set_wt_op(my_operator *w){wt=w;}
    void set_m0(my_vector *m){ m0=m;}
    virtual void setup_solver(){assert(1==2);}
    void init_lin_solver();
    
    void set_solver_defaults();
    void create_wt_op();
    void create_solver_vecs();
    virtual norm_vector *init_rr();
    virtual void update_model(norm_vector *mod);
    virtual my_vector *return_model();
    
  
    norm_vector *g,*m,*gg,*rr,*d,*rw;
    my_operator *oper,*wt,*iop,*mask_op,*o_p,*wt_op;
    my_vector *m0;
    int verb;
    bool have_op,have_wt;
    step *st;
    
  private:
    void clean_up();
    double scale;
    
    
    

};
class simple_solver:public lin_solver{
  public:
    simple_solver(){}
    simple_solver(step *s,norm_vector *data, my_operator *op);
    virtual void setup_solver();
    ~simple_solver(){};
  

  private:
      void clean_up();

};
class reg_solver:public lin_solver{
  public:
    reg_solver(){set_reg_defaults();};
    reg_solver(step *s,norm_vector *data, my_operator *op,my_operator *reg, float eps);
    virtual void setup_solver();
    void set_reg0(norm_vector *r){res_reg=r;}
    void set_reg_defaults();  
    void set_eps(float e){eps=e;}
		void set_rop(my_operator *o){rop=o;}
    ~reg_solver(){ if(rop_chain!=0) delete rop_chain;};
    
  private:
    void clean_up();
    float eps;
    my_operator *rop,*rop_chain;
    norm_vector *res_reg;
};
class prec_solver:public lin_solver{
  public:
    prec_solver(){mask_prec=0;}
    prec_solver(step *s,norm_vector *data, my_operator *op,my_operator *reg, float eps);
     void set_p0(norm_vector *p){p0=p;}
     void set_res_prec(norm_vector *r){res_prec=r;}
     void set_prec_defaults();
     void set_pmask(my_operator *p){pmask_op=p;}
     virtual void setup_solver();
     void set_eps_wt(my_operator *e){eps_op=e;}


    ~prec_solver(){clean_up();}
      virtual void update_model(norm_vector *mod);
      virtual my_vector *return_model();
      virtual norm_vector *init_rr();
  private:
    void clean_up();
    float eps;
    my_operator *pop,*prec_chain_op,*pmask_op;
    my_operator *eps_op,*mask_prec;
    norm_vector *p0,*res_prec;
};
#endif
