#include "sep_params.h"


int sep_params::get_int(const char *arg) const{
  int ret;

  char msg[] = "Couldn't grab %s\n";
  if(0==getch(arg,"d",&ret)) seperr(msg,arg);
  return ret;
}
int sep_params::get_int(const char *arg, int def) const
{
  int ret;

  int ierr=getch(arg,"d",&ret);
  if(ierr==0) ret=def;

  return ret;
}

int *sep_params::get_ints(const char *arg,int nmax) const{
  int *ret=new int[nmax];

  char msg[] = "Couldn't grab %s\n";
  if(0==getch(arg,"d",ret)) seperr(msg,arg);
  return ret;
}
int *sep_params::get_ints(const char *arg, int nmax,int *def) const
{
  int *ret=new int[nmax];

  int ierr=getch(arg,"d",ret);
  for(int i=ierr; i < nmax; i++) ret[i]=def[i];

  return ret;
}

std::vector<int> sep_params::get_vints(const char *arg, int nmax) const
{
  int *ret=new int[nmax];
  std::vector<int> retv;

  int ierr = getch(arg,"d",ret);
  for(int i = 0; i < ierr; ++i) retv.push_back(ret[i]);
  return retv;
}

std::vector<int> sep_params::get_vints(const char *arg, int nmax, std::vector<int> def) const
{
  int *ret=new int[nmax];
  std::vector<int> retv;

  int ierr = getch(arg,"d",ret);
  for(int i = ierr; i < nmax; ++i) {
    if(def.empty()) seperr("get_vints: must provide a non-null empty vector\n");
    retv.push_back(def[i]);
  }
  return retv;
}

std::vector<int> sep_params::get_auxvints(const char *arg, int nmax, const char *tag) const
{
  int *ret=new int[nmax];
  std::vector<int> retv;

  int ierr = auxpar(arg,"d",ret,tag);
  for(int i = 0; i < ierr; ++i) retv.push_back(ret[i]);
  return retv;
}

std::vector<int> sep_params::get_auxvints(const char *arg, int nmax, const char *tag, std::vector<int> def) const
{
  int *ret=new int[nmax];
  std::vector<int> retv;

  int ierr = auxpar(arg,"d",ret,tag);
  for(int i = ierr; i < nmax; ++i) {
    retv.push_back(def[i]);
  }
  return retv;
}

float sep_params::get_float(const char *arg) const
{
  float ret;

  char msg[] = "Couldn't grab %s\n";
  if(0==getch(arg,"f",&ret)) seperr(msg,arg);
  return ret;
}
float sep_params::get_float(const char *arg, float def) const
{
  float ret;
  if(0==getch(arg,"f",&ret)) ret=def;
  return ret;
}

float *sep_params::get_floats(const char *arg, int nmax) const
{
  float *ret=new float[nmax];

  char msg[] = "Couldn't grab %s\n";
  if(0==getch(arg,"f",ret)) seperr(msg,arg);
  return ret;
}

float *sep_params::get_floats(const char *arg, int nmax,float *def) const
{
  float *ret=new float[nmax];

  int ierr=getch(arg,"f",ret);
  for(int i=ierr; i < nmax; i++) ret[i]=def[i];

  return ret;
}

std::vector<float> sep_params::get_vfloats(const char *arg, int nmax) const
{
  float *ret=new float[nmax];
  std::vector<float> retv;

  int ierr = getch(arg,"f",ret);
  for(int i = 0; i < ierr; ++i) retv.push_back(ret[i]);
  return retv;
}

std::vector<float> sep_params::get_vfloats(const char *arg, int nmax, std::vector<float> def) const
{
  float *ret=new float[nmax];
  std::vector<float> retv;

  int ierr = getch(arg,"f",ret);
  for(int i = 0; i < ierr; ++i) {
    retv.push_back(ret[i]);
  }
  for(int i = ierr; i < nmax; ++i) {
    retv.push_back(def[i]);
  }
  return retv;
}

std::vector<float> sep_params::get_auxvfloats(const char *arg, int nmax, const char *tag) const
{
  float *ret=new float[nmax];
  std::vector<float> retv;

  int ierr = auxpar(arg,"f",ret,tag);
  for(int i = 0; i < ierr; ++i) retv.push_back(ret[i]);
  return retv;
}

std::vector<float> sep_params::get_auxvfloats(const char *arg, int nmax, const char *tag, std::vector<float> def) const
{
  float *ret=new float[nmax];
  std::vector<float> retv;

  int ierr = auxpar(arg,"f",ret,tag);
  for(int i = 0; i < ierr; ++i) {
    retv.push_back(ret[i]);
  }
  for(int i = ierr; i < nmax; ++i) {
    retv.push_back(def[i]);
  }
  return retv;
}

std::string sep_params::get_string(const char *arg) const
{
  char x[1024];
  std::string ret;

  char msg[] = "Couldn't grab %s\n";
  if(0==getch(arg,"s",x)) seperr(msg,arg);
  ret=x;
  return ret;
}
std::string sep_params::get_string(const char *arg, const char *def) const
{
  char x[1024];
  std::string ret;
  int iret=getch(arg,"s",x);
  if(iret==0) ret=def;
  else ret=x;
  return ret;
}
void sep_params::error(const std::string err) const
{
  seperr("%s",err.c_str());

}
bool sep_params::is_input_file(const std::string tag) const
{
  FILE *junk;
  junk=auxin(tag.c_str());
  if (junk==NULL) return false;
  return true;

}
bool sep_params::is_inout_file(const std::string tag) const
{
  FILE *junk;
  junk=auxinout(tag.c_str());
  if (junk==NULL) return false;
  return true;

}



