/**
 * Acoustic, constant-density,
 * finite difference wave propagation
 * operator
 * @author: Joseph Jennings
 * @version: 2016.11.26
 */
#ifndef ACFDPROP2D_H_
#define ACFDPROP2D_H_

#include <math.h>
#include "hypercube_float.h"
#include "sregf.h"
#include "laplacian.h"
#include "fdcoef.h"

class acfdprop2d : public my_operator {
  public:
    acfdprop2d(){};
    acfdprop2d(std::vector<axis> uaxis, hypercube_float *vel, hypercube_float *wgt, hypercube_float *vwt, std::vector<int> nbs, laplacian* lapop, float fmax);
    acfdprop2d(std::vector<axis> uaxis, hypercube_float *vel, laplacian* lapop, float fmax);
    bool forward(bool add, my_vector *model, my_vector *data);
    bool adjoint(bool add, my_vector *model, my_vector *data);
    void set_wfldout(std::string tag);
    float get_cfl();
    float get_ptsperlength();
    ~acfdprop2d(){
      delete _v2dt2; delete _lapop; delete _slc; delete _coefop;
      delete _dlc;
    };

  private:
    hypercube_float *_v2dt2, *_dlc, *_slc, *_w;
    laplacian *_lapop;
    fdcoef *_coefop;
    sregf _wuout;
    std::vector<int> _sn;
    int _nx, _nz, _nt;
    float _dx, _dz, _dt;
    float _vmin, _vmax;
    float _fmax;
    static constexpr float _CFL = sqrt(2)/2;
    float _mycfl, _ppl;
    float _T2SX;
    float find_vmin(hypercube_float *vel);
    bool check_cfl(float vmin);
    bool check_ptsperlength(float vmax, float fmax);
    bool check_dxdz(laplacian* lapop);
    void set_start(int start);
};

#endif /* ACFDPROP2D_H_ */
