/**
 * Constant-density, acoustic finite
 * difference wave propagation (fast)
 * @author: Joseph Jennings
 * @version: 2017.01.21
 */
#ifndef AFD2DFST_H_
#define AFD2DFST_H_

#include <vector>
#include <math.h>
#include "my_operator.h"
#include "hypercube_float.h"
#include "laplacian.h"
#include "tapercos.h"

class afd2dfst : my_operator {

  public:
    afd2dfst();
    afd2dfst(std::vector<axis> uaxes, hypercube_float *vel, laplacian* lapop, float dtu, float fmax);
    afd2dfst(std::vector<axis> uaxes, hypercube_float *vel, laplacian* lapop, float dtu, float fmax, int bz, int bx, float alpha);
    bool forward(bool add, my_vector *model, my_vector *data);
    bool adjoint(bool add, my_vector *model, my_vector *data);
    void set_bndpars(int bz, int bx, float alpha);
    std::pair<float,float> get_cfl();
    float get_ptsperlength();
    ~afd2dfst(){
      delete _v2dt2;
      delete _pre; delete _cur; delete _nex;
      delete _sou1; delete _lap;
      delete _sou2;
    };

  private:
    hypercube_float *_v2dt2, *_pre, *_cur, *_nex, *_sou1, *_sou2, *_tmp, *_lap, *_tsou1, *_tsou2;
    laplacian *_lapop;
    tapercos *_spg;
    std::vector<int> _sn;
    std::vector<double> _ls1, _ls2, _lr1, _lr2;
    int _nx, _nz, _ntu, _nts, _sample;
    int _onestp;
    float _dx, _dz, _dtu, _dts;
    float _vmin, _vmax;
    float _fmax;
    float _CFL;
    float _mycfl, _ppl;
    float _T2SX;
    void init(std::vector<axis> uaxes, hypercube_float *vel, laplacian *lapop, float dtu, float fmax, int bz, int bx, float alpha);
    float find_vmin(hypercube_float *vel);
    bool check_cfl(float vmax, laplacian *lapop);
    bool check_ptsperlength(float vmin, float fmax, laplacian *lapop);
    bool check_dxdz(laplacian* lapop);
    std::pair<std::vector<double>,std::vector<double>>interp_coeff(int sample);
    std::pair<std::vector<double>,std::vector<double>>interp_coefa(int sample);
    void fmarch(hypercube_float *pre, hypercube_float *cur, hypercube_float *nex);
    void bmarch(hypercube_float *pre, hypercube_float *cur, hypercube_float *nex);
};

#endif /* AFD2DFST_H_ */
