/**
 * 3D constant density, acoustic
 * finite difference wave equation
 * solver
 * @author: Joseph Jennings
 * @version: 2017.01.14
 */
#ifndef AFD3D_H_
#define AFD3D_H_

#include <vector>
#include <math.h>
#include "my_operator.h"
#include "hypercube_float.h"
#include "axis.h"
#include "laplacian3d.h"

class afd3d : public my_operator {
  public:
    afd3d(){};
    afd3d(std::vector<axis> uaxis, hypercube_float *vel, laplacian3d* lapop, float fmax);
    //afd3d(std::vector<axis> uaxis, std::vector<axis> saxis, hypercube_float *vel, laplacian* lapop, float fmax);
    //afd3d(std::vector<axis> uaxis, std::vector<axis> saxis, hypercube_float *vel, laplacian* lapop, float fmax, int bz, int bx, int by, float alpha);
    bool forward(bool add, my_vector *model, my_vector *data);
    bool adjoint(bool add, my_vector *model, my_vector *data);
    void set_bndpars(int bz, int bx, int by, float alpha);
    std::pair<float,float> get_cfl();
    float get_ptsperlength();
    ~afd3d(){
      delete _v2dt2; delete _tmp; delete _lap;
      delete _pre; delete _cur; delete _nex;
    };
  private:
    hypercube_float *_v2dt2, *_pre, *_cur, *_nex, *_tmp, *_lap, *_sou;
    laplacian3d* _lapop;
    int _nx, _nz, _ny, _ntu;
    int _onestp;
    float _dx, _dz, _dy, _dtu;
    float _vmin, _vmax;
    float _fmax;
    float _CFL;
    float _mycfl, _ppl;
    float _T2SX;
    //void init(std::vector<axis> uaxis, std::vector<axis> saxis, hypercube_float *vel, laplacian *lapop, float fmax, int bz, int bx, int by, float alpha);
    float find_vmin(hypercube_float *vel);
    bool check_cfl(float vmax, laplacian3d* lapop);
    bool check_ptsperlength(float vmin, float fmax, laplacian3d* lapop);
    bool check_dxdz(laplacian3d* lapop);
    void fmarch(hypercube_float *pre, hypercube_float *cur, hypercube_float *nex);
    void bmarch(hypercube_float *pre, hypercube_float *cur, hypercube_float *nex);
};

#endif /* AFD3D_H_ */
