/*
 * bcder.cpp
 *
 *  Created on: Nov 27, 2016
 *      Author: joe
 */

#include "bcder.h"
#include "hypercube_float.h"

bcder::bcder(std::vector<axis> uaxis, std::vector<int> nbs, int order) {
  _nz = uaxis[0].n; _bz1 = nbs[0]; _bz2 = nbs[2];
  _nx = uaxis[1].n; _bx1 = nbs[1]; _bx2 = nbs[3];
  _shift = order/2;
}

//TODO: Perhaps this is so slow because of fast and slow axes?
//TODO: Why is the adjoint so much faster than the forward?
bool bcder::forward(bool add, my_vector *mod, my_vector *dat) {
  if(!add && dat) dat->zero();

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  /* Foward difference in z */
  for(int iz = _shift; iz < _bz1 + _shift; ++iz) {
    for(int ix = _shift; ix < _nx - _shift; ++ix) {
      d->vals[ix*_nz + iz] += m->vals[ix*_nz + iz] - m->vals[ix*_nz + iz+1];
    }
  }
  /* Backward difference in z */
  for(int iz =_nz -_bz2-_shift; iz < _nz - _shift; ++iz) {
    for(int ix = _shift; ix < _nx - _shift; ++ix) {
      d->vals[ix*_nz + iz] += m->vals[ix*_nz + iz] - m->vals[ix*_nz + iz-1];
    }
  }
  /* Forward difference in x */
  for(int ix = _shift; ix < _bx1 + _shift; ++ix) {
    for(int iz = _shift; iz < _nz - _shift; ++iz) {
      d->vals[ix*_nz + iz] += m->vals[ix*_nz + iz] - m->vals[(ix+1)*_nz + iz];
    }
  }
  /* Backward difference in x */
  for(int ix = _nx - _bx2 - _shift; ix < _nx - _shift; ++ix) {
    for(int iz = _shift; iz < _nz - _shift; ++iz) {
      d->vals[ix*_nz + iz] += m->vals[ix*_nz + iz] - m->vals[(ix-1)*_nz + iz];
    }
  }

  return true;
}

bool bcder::adjoint(bool add, my_vector *mod, my_vector *dat) {
  if(!add && mod) mod->zero();

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  /* Endpoint iz=0*/
  for(int ix = _shift; ix < _nx - _shift; ++ix) {
    m->vals[ix*_nz + _shift] += d->vals[ix*_nz + _shift];
  }
  /* iz first boundary */
  for(int iz = 1 + _shift; iz < _bz1 + _shift; ++iz) {
    for(int ix = _shift; ix < _nx - _shift; ++ix) {
      m->vals[ix*_nz + iz] += d->vals[ix*_nz + iz] - d->vals[ix*_nz + iz-1];
    }
  }
  /* Endpoint iz=_bz1 */
  for(int ix = _shift; ix < _nx - _shift; ++ix) {
    m->vals[ix*_nz + _bz1 + _shift] += -d->vals[ix*_nz + _bz1 + _shift - 1];
  }

  /* Endpoint iz=_bz2 */
  for(int ix = _shift; ix < _nx - _shift; ++ix) {
    m->vals[ix*_nz + _nz - _bz2 - _shift - 1] += -d->vals[ix*_nz + _nz - _bz2 - _shift];
  }
  /* iz second boundary */
  for(int iz =_nz - _bz2 - _shift; iz < _nz - 1 -_shift; ++iz) {
    for(int ix = _shift; ix < _nx - _shift; ++ix) {
      m->vals[ix*_nz + iz] += d->vals[ix*_nz + iz] - d->vals[ix*_nz + iz+1];
    }
  }
  /* Endpoint iz=nz */
  for(int ix = _shift; ix < _nx - _shift; ++ix) {
    m->vals[ix*_nz + _nz - 1 -_shift] += d->vals[ix*_nz + _nz - 1 - _shift];
  }

  /* Endpoint ix=0 */
  for(int iz = _shift; iz < _nz - _shift; ++iz) {
    m->vals[_shift*_nz + iz] += d->vals[_shift*_nz + iz];
  }
  /* ix first boundary*/
  for(int ix = 1 + _shift; ix < _bx1 + _shift; ++ix) {
    for(int iz = _shift; iz < _nz - _shift; ++iz) {
      m->vals[ix*_nz + iz] += d->vals[ix*_nz + iz] - d->vals[(ix-1)*_nz + iz];
    }
  }
  /* Endpoint ix=_bx1 */
  for(int iz = _shift; iz < _nz - _shift; ++iz) {
    m->vals[(_bx1 + _shift)*_nz + iz] += -d->vals[(_bx1 + _shift - 1)*_nz + iz];
  }

  /* Endpoint ix=_bx2 */
  for(int iz = _shift; iz < _nz - _shift; ++iz) {
    m->vals[(_nx - _bx2 - _shift - 1)*_nz + iz] += -d->vals[(_nx - _bx2 - _shift)*_nz + iz];
  }
  /* ix second boundary */
  for(int ix = _nx - _bx2 - _shift; ix < _nx-1-_shift; ++ix) {
    for(int iz = _shift; iz < _nz - _shift; ++iz) {
      m->vals[ix*_nz + iz] += d->vals[ix*_nz + iz] - d->vals[(ix+1)*_nz + iz];
    }
  }
  /* Endpoint ix=nx */
  for(int iz = _shift; iz < _nz - _shift; ++iz) {
    m->vals[(_nx - 1 - _shift)*_nz + iz] += d->vals[(_nx - 1 - _shift)*_nz + iz];
  }

  return true;
}
