/*
 * bscat.cpp
 *
 *  Created on: Dec 28, 2016
 *      Author: joe
 */

#include "bscat.h"

bscat::bscat(hypercube_float *u0) {
  _u0 = u0;
  _nz = u0->get_axis(1).n;
  _nx = u0->get_axis(2).n;
  _nt = u0->get_axis(3).n;
}

bool bscat::forward(bool add, my_vector *mod, my_vector *dat) {
  if(!add && dat) dat->zero();

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  if(d->get_axis(3).n != _nt) {
    seperr("Data and background wavefield must have same temporal axes\n.");
  }

  for(int it = 0; it < _nt; ++it) {
    for(int ix = 0; ix < _nx; ++ix) {
      for(int iz = 0; iz < _nz; ++iz) {
        d->vals[it*_nx*_nz + ix*_nz + iz] += m->vals[ix*_nz + iz] * _u0->vals[it*_nx*_nz + ix*_nz + iz];
      }
    }
  }

  return true;
}

bool bscat::adjoint(bool add, my_vector *mod, my_vector *dat) {
  if(!add && mod) mod->zero();

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  if(d->get_axis(3).n != _nt) {
    seperr("Data and background wavefield must have same temporal axes\n.");
  }

  for(int it = 0; it < _nt; ++it) {
    for(int ix = 0; ix < _nx; ++ix) {
      for(int iz = 0; iz < _nz; ++iz) {
        m->vals[ix*_nz + iz] += d->vals[it*_nx*_nz + ix*_nz + iz] * _u0->vals[it*_nx*_nz + ix*_nz + iz];
      }
    }
  }

  return true;
}
