/*
 * fdcoefb.cpp
 *
 *  Created on: Nov 27, 2016
 *      Author: joe
 */

#include "fdcoefb.h"

fdcoefb::fdcoefb(std::vector<axis> azx, laplacian* lapop, hypercube_float *v2dt2,
    hypercube_float *wgt, hypercube_float *vwt, std::vector<int> nbs) {
  _nz = azx[0].n; _nx = azx[1].n;
  _lapop = lapop; _v2dt2 = v2dt2;
  _wgt = wgt; _vwt = vwt;
  _bdop = bcder(azx, nbs, _lapop->get_myorder());
  _lap  = new hypercube_float(azx, true);
  _bder = new hypercube_float(azx, true);
}

bool fdcoefb::forward(bool add, my_vector *mod, my_vector *dat) {
  if(!add && dat) dat->zero();

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  /* Apply laplacian */
  _lapop->forward(false,m,_lap);

  /* Apply boundary derivative */
  _bdop.forward(false,m,_bder);

  for(int ix = 0; ix < _nx; ++ix) {
    for(int iz = 0; iz < _nz; ++iz) {
      float wt  = (1-_wgt->vals[ix*_nz + iz]);
      d->vals[ix*_nz + iz] += wt*(_v2dt2->vals[ix*_nz + iz]*_lap->vals[ix*_nz + iz]) + 2*wt*m->vals[ix*_nz + iz];
      d->vals[ix*_nz + iz] += _wgt->vals[ix*_nz + iz]*(m->vals[ix*_nz + iz] - _vwt->vals[ix*_nz + iz]*_bder->vals[ix*_nz + iz]);
    }
  }

  return true;
}

bool fdcoefb::adjoint(bool add, my_vector *mod, my_vector *dat) {
  if(!add && mod) mod->zero();

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  /* _lap = v²Δt²(I-w)d */
  for(int ix = 0; ix < _nx; ++ix) {
    for(int iz = 0; iz < _nz; ++iz) {
      float wt = 1 - _wgt->vals[ix*_nz + iz];
      _lap->vals[ix*_nz + iz] = _v2dt2->vals[ix*_nz + iz] * wt * d->vals[ix*_nz + iz];
    }
  }

  /* m₁ = ∇²_lap */
  _lapop->adjoint(false,m,_lap);

  /* m₁ = m₁ + 2I(I-w)d*/
  for(int ix = 0; ix < _nx; ++ix) {
    for(int iz = 0; iz < _nz; ++iz) {
      float wt = 2 - _wgt->vals[ix*_nz + iz];
      m->vals[ix*_nz + iz] +=  wt * d->vals[ix*_nz + iz];
    }
  }

  /* _bder = vΔtwd */
  for(int ix = 0; ix < _nx; ++ix) {
    for(int iz = 0; iz < _nz; ++iz) {
      _bder->vals[ix*_nz + iz] = -_vwt->vals[ix*_nz + iz] * _wgt->vals[ix*_nz + iz] * d->vals[ix*_nz + iz];
    }
  }

  /* m = m₁ + ∂/∂s(_bder) */
  _bdop.adjoint(true,m,_bder);

  return true;
}
