/*
 * fdcoefnb.cpp
 *
 *  Created on: Nov 27, 2016
 *      Author: joe
 */

#include "fdcoefnb.h"

fdcoefnb::fdcoefnb(std::vector<axis> uaxis, laplacian *lapop, hypercube_float* v2dt2) {
  _nz = uaxis[0].n; _nx = uaxis[1].n;
  _lapop = lapop; _v2dt2 = v2dt2;
  _lap = new hypercube_float(uaxis);
}

bool fdcoefnb::forward(bool add, my_vector *mod, my_vector *dat) {

  if(!mod->check_match(dat)) {
    seperr("Wavefield slices must have same axes (nz,nx).\n");
  }

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  /* Apply laplacian */
  _lapop->forward(false,m,_lap);

  for(int ix = 0; ix < _nx; ++ix) {
    for(int iz = 0; iz < _nz; ++iz) {
      d->vals[ix*_nz + iz] = _v2dt2->vals[ix*_nz + iz]*_lap->vals[ix*_nz + iz] + 2*m->vals[ix*_nz + iz];
    }
  }
  return true;
}

bool fdcoefnb::adjoint(bool add, my_vector *mod, my_vector *dat) {

  if(!dat->check_match(mod)) {
    seperr("Wavefield slices must have same axes (nz,nx).\n");
  }

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  for(int ix = 0; ix < _nx; ++ix) {
    for(int iz = 0; iz < _nz; ++iz) {
      _lap->vals[ix*_nz + iz] = _v2dt2->vals[ix*_nz + iz]*d->vals[ix*_nz + iz];
    }
  }

  /* Apply laplacian */
  _lapop->adjoint(false, m, _lap);

  for(int ix = 0; ix < _nx; ++ix) {
    for(int iz = 0; iz < _nz; ++iz) {
      m->vals[ix*_nz + iz] += 2*d->vals[ix*_nz + iz];
    }
  }

  return true;
}
