/*
 * igrad3.cpp
 *
 *  Created on: Jan 10, 2017
 *      Author: joe
 */

#include "igrad3.h"
#include "hypercube_float.h"

igrad3::igrad3(std::vector<axis> uaxes) {
  if((int)uaxes.size() != 4)
    seperr("igrad3: Input wavefield must be three dimensional.\n");
  _nz = uaxes[0].n; _idz = 1/uaxes[0].d;
  _nx = uaxes[1].n; _idx = 1/uaxes[1].d;
  _ny = uaxes[2].n; _idy = 1/uaxes[2].d;
  _nt = uaxes[3].n;
}

bool igrad3::forward(bool add, my_vector *mod, my_vector *dat) {
  if(!add && dat) dat->zero();

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  if(d->get_axis(5).n != 3)
    seperr("igrad3: data n5 must equal 3 in order to hold each component.\n");

  for(int it = 0; it < _nt; ++it) {
    for(int iy = 0; iy < _ny-1; ++iy) {
      for(int ix = 0; ix < _nx-1; ++ix) {
        for(int iz = 0; iz < _nz-1; ++iz) {
          float val = m->vals[it*_nz*_nx*_ny + iy*_nz*_nx + ix*_nz + iz];
          /* Derivative in z */
          d->vals[0*_nz*_nx*_ny*_nt + it*_nz*_nx*_ny + iy*_nz*_nx + ix*_nz + iz] +=
              (m->vals[it*_nz*_nx*_ny + iy*_nz*_nx + ix*_nz + (iz+1)] - val)*_idz;
          /* Derivative in x */
          d->vals[1*_nz*_nx*_ny*_nt + it*_nz*_nx*_ny + iy*_nz*_nx + ix*_nz + iz] +=
              (m->vals[it*_nz*_nx*_ny + iy*_nz*_nx + (ix+1)*_nz + iz] - val)*_idx;
          /* Derivative in y */
          d->vals[2*_nz*_nx*_ny*_nt + it*_nz*_nx*_ny + iy*_nz*_nx + ix*_nz + iz] +=
              (m->vals[it*_nz*_nx*_ny + (iy+1)*_nz*_nx + ix*_nz + iz] - val)*_idy;
        }
      }
    }
  }
  return true;
}

bool igrad3::adjoint(bool add, my_vector *mod, my_vector *dat) {
  if(!add && mod) mod->zero();
  return true;
}
