/*
 * laplacian3d10th.cpp
 *
 *  Created on: Jan 8, 2017
 *      Author: joe
 */

#include "laplacian3d10th.h"
#include "hypercube_float.h"

laplacian3d10th::laplacian3d10th(std::vector<axis> uaxes) {
  if(uaxes.size() < 3) seperr("Must apply 3D laplacian to vector with dim > 2.\n");
  _idz2 = 1/(uaxes[0].d * uaxes[0].d); _idx2 = 1/(uaxes[1].d * uaxes[1].d);
  _idy2 = 1/(uaxes[2].d * uaxes[2].d);
  _nz = uaxes[0].n; _nx = uaxes[1].n; _ny = uaxes[2].n;
  _coefs = _coefs10;
  _myorder = _coefs.size() * 2 - 2;
}

bool laplacian3d10th::forward(bool add, my_vector *mod, my_vector *dat) {
  if(!add && dat) dat->zero();

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  for(int iy = 5; iy < _ny-5; ++iy) {
    for(int ix = 5; ix < _nx-5; ++ix) {
      for(int iz = 5; iz < _nz-5; ++iz) {
        d->vals[iy*_nz*_nx + ix*_nz + iz] += _coefs[0] * m->vals[iy*_nz*_nx + ix*_nz + iz]*(_idz2 + _idx2 + _idy2) +
        _coefs[1]*(m->vals[(iy  )*_nz*_nx + (ix  )*_nz + iz+1] + m->vals[(iy  )*_nz*_nx + (ix  )*_nz + iz-1])*_idz2 +
        _coefs[2]*(m->vals[(iy  )*_nz*_nx + (ix  )*_nz + iz+2] + m->vals[(iy  )*_nz*_nx + (ix  )*_nz + iz-2])*_idz2 +
        _coefs[3]*(m->vals[(iy  )*_nz*_nx + (ix  )*_nz + iz+3] + m->vals[(iy  )*_nz*_nx + (ix  )*_nz + iz-3])*_idz2 +
        _coefs[4]*(m->vals[(iy  )*_nz*_nx + (ix  )*_nz + iz+4] + m->vals[(iy  )*_nz*_nx + (ix  )*_nz + iz-4])*_idz2 +
        _coefs[5]*(m->vals[(iy  )*_nz*_nx + (ix  )*_nz + iz+5] + m->vals[(iy  )*_nz*_nx + (ix  )*_nz + iz-5])*_idz2 +
        _coefs[1]*(m->vals[(iy  )*_nz*_nx + (ix+1)*_nz + iz  ] + m->vals[(iy  )*_nz*_nx + (ix-1)*_nz + iz  ])*_idx2 +
        _coefs[2]*(m->vals[(iy  )*_nz*_nx + (ix+2)*_nz + iz  ] + m->vals[(iy  )*_nz*_nx + (ix-2)*_nz + iz  ])*_idx2 +
        _coefs[3]*(m->vals[(iy  )*_nz*_nx + (ix+3)*_nz + iz  ] + m->vals[(iy  )*_nz*_nx + (ix-3)*_nz + iz  ])*_idx2 +
        _coefs[4]*(m->vals[(iy  )*_nz*_nx + (ix+4)*_nz + iz  ] + m->vals[(iy  )*_nz*_nx + (ix-4)*_nz + iz  ])*_idx2 +
        _coefs[5]*(m->vals[(iy  )*_nz*_nx + (ix+5)*_nz + iz  ] + m->vals[(iy  )*_nz*_nx + (ix-5)*_nz + iz  ])*_idx2 +
        _coefs[1]*(m->vals[(iy+1)*_nz*_nx + (ix  )*_nz + iz  ] + m->vals[(iy-1)*_nz*_nx + (ix  )*_nz + iz  ])*_idy2 +
        _coefs[2]*(m->vals[(iy+2)*_nz*_nx + (ix  )*_nz + iz  ] + m->vals[(iy-2)*_nz*_nx + (ix  )*_nz + iz  ])*_idy2 +
        _coefs[3]*(m->vals[(iy+3)*_nz*_nx + (ix  )*_nz + iz  ] + m->vals[(iy-3)*_nz*_nx + (ix  )*_nz + iz  ])*_idy2 +
        _coefs[4]*(m->vals[(iy+4)*_nz*_nx + (ix  )*_nz + iz  ] + m->vals[(iy-4)*_nz*_nx + (ix  )*_nz + iz  ])*_idy2 +
        _coefs[5]*(m->vals[(iy+5)*_nz*_nx + (ix  )*_nz + iz  ] + m->vals[(iy-5)*_nz*_nx + (ix  )*_nz + iz  ])*_idy2;
      }
    }
  }

  return true;
}

bool laplacian3d10th::adjoint(bool add, my_vector *mod, my_vector *dat) {
  if(!add && mod) mod->zero();

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  for(int iy = 5; iy < _ny-5; ++iy) {
    for(int ix = 5; ix < _nx-5; ++ix) {
      for(int iz = 5; iz < _nz-5; ++iz) {
        m->vals[iy*_nz*_nx + ix*_nz + iz] += _coefs[0] * d->vals[iy*_nz*_nx + ix*_nz + iz]*(_idz2 + _idx2 + _idy2) +
        _coefs[1]*(d->vals[(iy  )*_nz*_nx + (ix  )*_nz + iz+1] + d->vals[(iy  )*_nz*_nx + (ix  )*_nz + iz-1])*_idz2 +
        _coefs[2]*(d->vals[(iy  )*_nz*_nx + (ix  )*_nz + iz+2] + d->vals[(iy  )*_nz*_nx + (ix  )*_nz + iz-2])*_idz2 +
        _coefs[3]*(d->vals[(iy  )*_nz*_nx + (ix  )*_nz + iz+3] + d->vals[(iy  )*_nz*_nx + (ix  )*_nz + iz-3])*_idz2 +
        _coefs[4]*(d->vals[(iy  )*_nz*_nx + (ix  )*_nz + iz+4] + d->vals[(iy  )*_nz*_nx + (ix  )*_nz + iz-4])*_idz2 +
        _coefs[5]*(d->vals[(iy  )*_nz*_nx + (ix  )*_nz + iz+5] + d->vals[(iy  )*_nz*_nx + (ix  )*_nz + iz-5])*_idz2 +
        _coefs[1]*(d->vals[(iy  )*_nz*_nx + (ix+1)*_nz + iz  ] + d->vals[(iy  )*_nz*_nx + (ix-1)*_nz + iz  ])*_idx2 +
        _coefs[2]*(d->vals[(iy  )*_nz*_nx + (ix+2)*_nz + iz  ] + d->vals[(iy  )*_nz*_nx + (ix-2)*_nz + iz  ])*_idx2 +
        _coefs[3]*(d->vals[(iy  )*_nz*_nx + (ix+3)*_nz + iz  ] + d->vals[(iy  )*_nz*_nx + (ix-3)*_nz + iz  ])*_idx2 +
        _coefs[4]*(d->vals[(iy  )*_nz*_nx + (ix+4)*_nz + iz  ] + d->vals[(iy  )*_nz*_nx + (ix-4)*_nz + iz  ])*_idx2 +
        _coefs[5]*(d->vals[(iy  )*_nz*_nx + (ix+5)*_nz + iz  ] + d->vals[(iy  )*_nz*_nx + (ix-5)*_nz + iz  ])*_idx2 +
        _coefs[1]*(d->vals[(iy+1)*_nz*_nx + (ix  )*_nz + iz  ] + d->vals[(iy-1)*_nz*_nx + (ix  )*_nz + iz  ])*_idy2 +
        _coefs[2]*(d->vals[(iy+2)*_nz*_nx + (ix  )*_nz + iz  ] + d->vals[(iy-2)*_nz*_nx + (ix  )*_nz + iz  ])*_idy2 +
        _coefs[3]*(d->vals[(iy+3)*_nz*_nx + (ix  )*_nz + iz  ] + d->vals[(iy-3)*_nz*_nx + (ix  )*_nz + iz  ])*_idy2 +
        _coefs[4]*(d->vals[(iy+4)*_nz*_nx + (ix  )*_nz + iz  ] + d->vals[(iy-4)*_nz*_nx + (ix  )*_nz + iz  ])*_idy2 +
        _coefs[5]*(d->vals[(iy+5)*_nz*_nx + (ix  )*_nz + iz  ] + d->vals[(iy-5)*_nz*_nx + (ix  )*_nz + iz  ])*_idy2;

      }
    }
  }

  return true;
}
