/*
 * laplacian3d4th.cpp
 *
 *  Created on: Jan 8, 2017
 *      Author: joe
 */

#include "laplacian3d4th.h"
#include "hypercube_float.h"

laplacian3d4th::laplacian3d4th(std::vector<axis> uaxes) {
  if(uaxes.size() < 3) seperr("Must apply 3D laplacian to vector with dim > 2.\n");
  _idz2 = 1/(uaxes[0].d * uaxes[0].d); _idx2 = 1/(uaxes[1].d * uaxes[1].d); _idy2 = 1/(uaxes[2].d * uaxes[2].d);
  _nz = uaxes[0].n; _nx = uaxes[1].n; _ny = uaxes[2].n;
  _coefs = _coefs4;
  _myorder = _coefs.size() * 2 - 2;
}

bool laplacian3d4th::forward(bool add, my_vector *mod, my_vector *dat) {
  if(!add && dat) dat->zero();

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  for(int iy = 2; iy < _ny-2; ++iy) {
    for(int ix = 2; ix < _nx-2; ++ix) {
      for(int iz = 2; iz < _nz-2; ++iz) {
        d->vals[iy*_nz*_nx + ix*_nz + iz] += _coefs[0] * m->vals[iy*_nz*_nx + ix*_nz + iz]*(_idz2 + _idx2 + _idy2) +
            _coefs[1]*(m->vals[(iy  )*_nz*_nx + (ix  )*_nz + iz+1] + m->vals[(iy  )*_nz*_nx + (ix  )*_nz + iz-1])*_idz2 +
            _coefs[2]*(m->vals[(iy  )*_nz*_nx + (ix  )*_nz + iz+2] + m->vals[(iy  )*_nz*_nx + (ix  )*_nz + iz-2])*_idz2 +
            _coefs[1]*(m->vals[(iy  )*_nz*_nx + (ix+1)*_nz + iz  ] + m->vals[(iy  )*_nz*_nx + (ix-1)*_nz + iz  ])*_idx2 +
            _coefs[2]*(m->vals[(iy  )*_nz*_nx + (ix+2)*_nz + iz  ] + m->vals[(iy  )*_nz*_nx + (ix-2)*_nz + iz  ])*_idx2 +
            _coefs[1]*(m->vals[(iy+1)*_nz*_nx + (ix  )*_nz + iz  ] + m->vals[(iy-1)*_nz*_nx + (ix  )*_nz + iz  ])*_idy2 +
            _coefs[2]*(m->vals[(iy+2)*_nz*_nx + (ix  )*_nz + iz  ] + m->vals[(iy-2)*_nz*_nx + (ix  )*_nz + iz  ])*_idy2;
      }
    }
  }
  return true;
}

bool laplacian3d4th::adjoint(bool add, my_vector *mod, my_vector *dat) {
  if(!add && mod) mod->zero();

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  for(int iy = 2; iy < _ny-2; ++iy) {
    for(int ix = 2; ix < _nx-2; ++ix) {
      for(int iz = 2; iz < _nz-2; ++iz) {
        m->vals[iy*_nz*_nx + ix*_nz + iz] += _coefs[0] * d->vals[iy*_nz*_nx + ix*_nz + iz]*(_idz2 + _idx2 + _idy2) +
            _coefs[1]*(d->vals[(iy  )*_nz*_nx + (ix  )*_nz + iz+1] + d->vals[(iy  )*_nz*_nx + (ix  )*_nz + iz-1])*_idz2 +
            _coefs[2]*(d->vals[(iy  )*_nz*_nx + (ix  )*_nz + iz+2] + d->vals[(iy  )*_nz*_nx + (ix  )*_nz + iz-2])*_idz2 +
            _coefs[1]*(d->vals[(iy  )*_nz*_nx + (ix+1)*_nz + iz  ] + d->vals[(iy  )*_nz*_nx + (ix-1)*_nz + iz  ])*_idx2 +
            _coefs[2]*(d->vals[(iy  )*_nz*_nx + (ix+2)*_nz + iz  ] + d->vals[(iy  )*_nz*_nx + (ix-2)*_nz + iz  ])*_idx2 +
            _coefs[1]*(d->vals[(iy+1)*_nz*_nx + (ix  )*_nz + iz  ] + d->vals[(iy-1)*_nz*_nx + (ix  )*_nz + iz  ])*_idy2 +
            _coefs[2]*(d->vals[(iy+2)*_nz*_nx + (ix  )*_nz + iz  ] + d->vals[(iy-2)*_nz*_nx + (ix  )*_nz + iz  ])*_idy2;
      }
    }
  }
  return true;
}
