/*
 * laplacian4th.cpp
 *
 *  Created on: Nov 25, 2016
 *      Author: joe
 */

#include "laplacian4th.h"
#include "hypercube_float.h"

laplacian4th::laplacian4th(std::vector<axis> uaxes) {
  if(uaxes.size() < 2) seperr("Must apply laplacian to vector with dim > 1.\n");
  _idz2 = 1/(uaxes[0].d * uaxes[0].d); _idx2 = 1/(uaxes[1].d * uaxes[1].d);
  _nz = uaxes[0].n; _nx = uaxes[1].n;
  _coefs = _coefs4;
  _myorder = _coefs.size() * 2 - 2;
}

bool laplacian4th::forward(bool add, my_vector* mod, my_vector *dat) {
  if(!add && dat) dat->zero();

  hypercube_float* m = (hypercube_float*)mod;
  hypercube_float* d = (hypercube_float*)dat;

  for(int ix = 2; ix < _nx - 2; ++ix) {
    for(int iz = 2; iz < _nz - 2; ++iz) {
      d->vals[ix*_nz + iz] += _coefs[0]* m->vals[(ix  )*_nz + iz  ] * (_idx2 + _idz2) +
          _coefs[1]*(m->vals[(ix  )*_nz + iz+1] + m->vals[(ix  )*_nz + iz-1])*_idz2 +
          _coefs[2]*(m->vals[(ix  )*_nz + iz+2] + m->vals[(ix  )*_nz + iz-2])*_idz2 +
          _coefs[1]*(m->vals[(ix+1)*_nz + iz  ] + m->vals[(ix-1)*_nz + iz  ])*_idx2 +
          _coefs[2]*(m->vals[(ix+2)*_nz + iz  ] + m->vals[(ix-2)*_nz + iz  ])*_idx2;
    }
  }

  return true;
}

bool laplacian4th::adjoint(bool add, my_vector *mod, my_vector *dat) {
  if(!add && mod) mod->zero();

  hypercube_float* m = (hypercube_float*)mod;
  hypercube_float* d = (hypercube_float*)dat;

  for(int ix = 2; ix < _nx - 2; ++ix) {
    for(int iz = 2; iz < _nz - 2; ++iz) {
      m->vals[ix*_nz + iz] += _coefs[0]* d->vals[(ix  )*_nz + iz  ] * (_idx2 + _idz2) +
          _coefs[1]*(d->vals[(ix  )*_nz + iz+1] + d->vals[(ix  )*_nz + iz-1])*_idz2 +
          _coefs[2]*(d->vals[(ix  )*_nz + iz+2] + d->vals[(ix  )*_nz + iz-2])*_idz2 +
          _coefs[1]*(d->vals[(ix+1)*_nz + iz  ] + d->vals[(ix-1)*_nz + iz  ])*_idx2 +
          _coefs[2]*(d->vals[(ix+2)*_nz + iz  ] + d->vals[(ix-2)*_nz + iz  ])*_idx2;
    }
  }
  return true;
}
