/*
 * rrec.cpp
 *
 *  Created on: Dec 27, 2016
 *      Author: joe
 */

#include "rrec.h"
#include "hypercube_float.h"

rrec::rrec(axis recx, int recz) {
  _rnx = recx.n; _rox = recx.o; _rdx = recx.d;
  _recz = recz;
}

bool rrec::forward(bool add, my_vector *mod, my_vector *dat) {
  if(!add && dat) dat->zero();

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  if(d->get_axis(3).n != m->get_axis(1).n) {
    seperr("rrec: Data and wavefield must have same temporal axis.\n");
  }

  int nz = d->get_axis(1).n;
  int nx = d->get_axis(2).n;
  int nt = d->get_axis(3).n;

  /* Find receiver location */
  for(int ir = 0; ir < _rnx; ++ir) {
    int recx = (ir)*_rdx + _rox;
    /* Copy all time samples for receiver location */
    for(int it = 0; it < nt; ++it) {
      d->vals[it*nx*nz + recx * nz + _recz] += m->vals[ir*nt + it];
    }
  }
  return true;
}

bool rrec::adjoint(bool add, my_vector *mod, my_vector *dat) {
  if(!add && mod) mod->zero();

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  if(d->get_axis(3).n != m->get_axis(1).n) {
    seperr("rrec: Data and wavefield must have same temporal axis.\n");
  }

  int nz = d->get_axis(1).n;
  int nx = d->get_axis(2).n;
  int nt = d->get_axis(3).n;

  /* Find receiver location */
  for(int ir = 0; ir < _rnx; ++ir) {
    int recx = (ir)*_rdx +  _rox;
    /* Copy all time samples for receiver location */
    for(int it = 0; it < nt; ++it) {
      m->vals[ir*nt + it] += d->vals[it*nx*nz + recx * nz + _recz];
    }
  }

  return true;
}

