/*
 * rrec3d.cpp
 *
 *  Created on: Jan 10, 2017
 *      Author: joe
 */

#include "rrec3d.h"
#include "hypercube_float.h"

rrec3d::rrec3d(std::vector<axis> uaxes, axis recx, axis recy, int recz) {
  _rnx = recx.n; _rox = recx.o; _rdx = recx.d;
  _rny = recy.n; _roy = recy.o; _rdy = recy.d;
  if(uaxes.size() < 4) {
    seperr("rrec3d only operates on 3D wavefields.\n");
  }
  _nz = uaxes[0].n; _nx = uaxes[1].n; _ny = uaxes[2].n; _nt = uaxes[3].n;
  int lastrx = (_rnx-1)*_rdx +_rox;
  if(lastrx >= _nx)
    seperr("Receiver nrx=%d cannot be greater than wavefield nx=%d\n",_rnx,_nx);
  int lastry = (_rnx-1)*_rdx +_rox;
  if(lastry >= _ny)
    seperr("Receiver nry=%d cannot be greater than wavefield ny=%d\n",_rnx,_ny);
  _recz = recz;
}

bool rrec3d::forward(bool add, my_vector *mod, my_vector *dat) {
  if(!add && dat) dat->zero();

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  if(_nt != m->get_axis(1).n)
    seperr("rrec3d: Data and wavefield must have same temporal axis.\n");

  /* Find receiver location */
  for(int iry = 0; iry < _rny; ++iry) {
    int recy = (iry)*_rdy + _roy;
    for(int irx = 0; irx < _rnx; ++irx) {
      int recx = (irx)*_rdx + _rox;
      /* Copy all samples for receiver location */
      for(int it = 0; it < _nt; ++it) {
        d->vals[it*_nx*_nz*_ny + recy*_nx*_nz + recx*_nz + _recz] += m->vals[iry*_nt*_rnx + irx*_nt + it];
      }
    }
  }
  return true;
}

bool rrec3d::adjoint(bool add, my_vector *mod, my_vector *dat) {
  if(!add && mod) mod->zero();

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  if(_nt != m->get_axis(1).n)
    seperr("rrec3d: Data and wavefield must have same temporal axis.\n");

  /* Find receiver location */
  for(int iry = 0; iry < _rny; ++iry) {
    int recy = (iry)*_rdy + _roy;
    for(int irx = 0; irx < _rnx; ++irx) {
      int recx = (irx)*_rdx + _rox;
      /* Copy all samples for receiver location */
      for(int it = 0; it < _nt; ++it) {
        m->vals[iry*_nt*_rnx + irx*_nt + it] += d->vals[it*_nx*_nz*_ny + recy*_nx*_nz + recx*_nz + _recz];
      }
    }
  }
  return true;
}
