/**
 * 3D receiver restriction operator
 * @author: Joseph Jennings
 * @version: 2017.01.10
 */
#ifndef RREC3D_H_
#define RREC3D_H_

#include <vector>
#include "my_operator.h"
#include "axis.h"

class rrec3d : public my_operator {
  public:
    rrec3d(){};
    /**
     * Constructs a 3D receiver restriction operator
     * @param uaxis the wavefield axes
     * @param recx the receiver x axis
     * @param ay the wavefield  y axis
     * @param recy the receiver y axis
     * @param recz the receiver z location
     */
    rrec3d(std::vector<axis> uaxes, axis recx, axis recy, int recz);
    /**
     * Injects the data at the receivers
     * @param add flag whether to add to previous data
     * @param model the input data
     * @param data the output wavefield
     * @return status flag
     */
    bool forward(bool add, my_vector *model, my_vector *data);
    /**
     * Restricts the data to the receivers
     * @param add flag whether to add to previous data
     * @param model the input data
     * @param data the output wavefield
     * @return status flag
     */
    bool adjoint(bool add, my_vector *model, my_vector *data);
    ~rrec3d(){};
  private:
    int _nz, _ny, _nx, _nt, _rnx, _rny;
    float _rox, _rdx, _roy, _rdy;
    int _recz;
};

#endif /* RREC3D_H_ */
