/*
 * scndder.cpp
 *
 *  Created on: Dec 28, 2016
 *      Author: joe
 */

#include "scndder.h"
#include "hypercube_float.h"

scndder::scndder(axis at) {
  _nt = at.n; _ot = at.o; _idt2 = 1.0/(at.d*at.d);
}

bool scndder::forward(bool add, my_vector *mod, my_vector *dat) {
  if(!add && dat) dat->zero();

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  m->check_same(d);

  if(m->get_axis(3).n != _nt) {
    seperr("Error in scndder: Time axes do not agree: model nt=%d cstr nt=%d",m->get_axis(3).n,_nt);
  }

  int nz = m->get_axis(1).n; int nx = m->get_axis(2).n;

  /* it = 0 */
  for(int ix = 0; ix < nx; ++ix) {
    for(int iz = 0; iz < nz; ++iz) {
      d->vals[ix*nx + iz] += (m->vals[nx*nz + ix*nz + iz] - m->vals[ix*nz + iz])*_idt2;
    }
  }

  /* Main time loop */
  for(int it = 1; it < _nt-1; ++it) {
    for(int ix = 0; ix < nx; ++ix) {
      for(int iz = 0; iz < nz; ++iz) {
        int itm = (it-1)*nx*nz + ix*nz + iz;
        int it0 = (it+0)*nx*nz + ix*nz + iz;
        int itp = (it+1)*nx*nz + ix*nz + iz;
        d->vals[it0] += (m->vals[itp] - 2 * m->vals[it0] + m->vals[itm])*_idt2;
      }
    }
  }

  /* it = nt-1 */
  for(int ix = 0; ix < nx; ++ix) {
    for(int iz = 0; iz < nz; ++iz) {
      d->vals[(_nt-1)*nx*nz + ix*nz + iz] += (m->vals[(_nt-2)*nx*nz + ix*nz + iz] - m->vals[(_nt-1)*nx*nz + ix*nz + iz])*_idt2;
    }
  }

  return true;
}

bool scndder::adjoint(bool add, my_vector *mod, my_vector *dat) {
  if(!add && mod) mod->zero();

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  m->check_same(d);

  if(m->get_axis(3).n != _nt) {
    seperr("Error in scndder: Time axes do not agree: model nt=%d cstr nt=%d",m->get_axis(3).n,_nt);
  }

  int nz = m->get_axis(1).n; int nx = m->get_axis(2).n;

  /* it = 0 */
  for(int ix = 0; ix < nx; ++ix) {
    for(int iz = 0; iz < nz; ++iz) {
      m->vals[ix*nx + iz] += (d->vals[nx*nz + ix*nz + iz] - d->vals[ix*nz + iz])*_idt2;
    }
  }

  /* Main time loop */
  for(int it = 1; it < _nt-1; ++it) {
    for(int ix = 0; ix < nx; ++ix) {
      for(int iz = 0; iz < nz; ++iz) {
        int itm = (it-1)*nx*nz + ix*nz + iz;
        int it0 = (it+0)*nx*nz + ix*nz + iz;
        int itp = (it+1)*nx*nz + ix*nz + iz;
        m->vals[it0] += (d->vals[itp] - 2 * d->vals[it0] + d->vals[itm])*_idt2;
      }
    }
  }

  /* it = nt-1 */
  for(int ix = 0; ix < nx; ++ix) {
    for(int iz = 0; iz < nz; ++iz) {
      m->vals[(_nt-1)*nx*nz + ix*nz + iz] += (d->vals[(_nt-2)*nx*nz + ix*nz + iz] - d->vals[(_nt-1)*nx*nz + ix*nz + iz])*_idt2;
    }
  }

  return true;
}

