/*
 * sourcescale.cpp
 *
 *  Created on: Nov 24, 2016
 *      Author: joe
 */

#include "sourcescale.h"

//TODO: You may not need to clone here. Need to think about it.
sourcescale::sourcescale(float dtu, hypercube_float *vel) {
  _v2dtu2 = (hypercube_float*)vel->clone_vec();
  _v2dtu2->mult(_v2dtu2);
  _v2dtu2->scale(dtu * dtu);
  _w = new hypercube_float(vel->get_axes(), true);
  _w->zero();
}

sourcescale::sourcescale(float dtu, hypercube_float *vel, hypercube_float *w) {
  _v2dtu2 = (hypercube_float*)vel->clone_vec();
  _v2dtu2->mult(_v2dtu2);
  _v2dtu2->scale(dtu * dtu);
  _w = w;
}

bool sourcescale::forward(bool add, my_vector *mod, my_vector* dat) {
  if(!add && dat) dat->zero();

  if(!mod->check_match(dat))
    seperr("sourcescale: padded source and scaled source must be same size.\n");

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  int nt = m->get_axis(3).n;
  int nx = m->get_axis(2).n;
  int nz = m->get_axis(1).n;

  for(int it = 0; it < nt; ++it) {
    for(int ix = 0; ix < nx; ++ix) {
      for(int iz = 0; iz < nz; ++iz) {
        int uidx = it*nx*nz + ix*nz + iz;
        int vidx =            ix*nz + iz;
        float wt = _w->vals[vidx]-1.0;
        d->vals[uidx] += wt*(_v2dtu2->vals[vidx] * m->vals[uidx]);
      }
    }
  }

  return true;
}

bool sourcescale::adjoint(bool add, my_vector *mod, my_vector *dat) {
  if(!add && mod) mod->zero();

  if(!mod->check_match(dat))
    seperr("sourcescale: padded source and scaled source must be same size.\n");

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  int nt = m->get_axis(3).n;
  int nx = m->get_axis(2).n;
  int nz = m->get_axis(1).n;

  for(int it = 0; it < nt; ++it) {
    for(int ix = 0; ix < nx; ++ix) {
      for(int iz = 0; iz < nz; ++iz) {
        int uidx = it*nx*nz + ix*nz + iz;
        int vidx =            ix*nz + iz;
        float wt = _w->vals[vidx]-1.0;
        m->vals[uidx] += wt*(_v2dtu2->vals[vidx] * d->vals[uidx]);
      }
    }
  }

  return true;
}
