/*
 * sourcescale3d.cpp
 *
 *  Created on: Jan 7, 2017
 *      Author: joe
 */

#include "sourcescale3d.h"

sourcescale3d::sourcescale3d(float dtu, hypercube_float *vel) {
  _v2dtu2 = (hypercube_float*)vel->clone_vec();
  _v2dtu2->mult(_v2dtu2);
  _v2dtu2->scale(dtu*dtu);
}

bool sourcescale3d::forward(bool add, my_vector* mod, my_vector *dat) {
  if(!add && dat) dat->zero();

  if(!mod->check_match(dat))
    seperr("sourcescale3d: padded source and scaled source must be same size.\n");

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  int nt = m->get_axis(4).n;
  int ny = m->get_axis(3).n;
  int nx = m->get_axis(2).n;
  int nz = m->get_axis(1).n;

  for(int it = 0; it < nt; ++it) {
    for(int iy = 0; iy < ny; ++iy) {
      for(int ix = 0; ix < nx; ++ix) {
        for(int iz = 0; iz < nz; ++iz) {
          int uidx = it*ny*nx*nz + iy*nx*nz + ix*nz + iz;
          int vidx =               iy*nx*nz + ix*nz + iz;
          d->vals[uidx] += -_v2dtu2->vals[vidx] * m->vals[uidx];
        }
      }
    }
  }

  return true;
}

bool sourcescale3d::adjoint(bool add, my_vector *mod, my_vector *dat) {
  if(!add && dat) dat->zero();

  if(!mod->check_match(dat))
    seperr("sourcescale3d: padded source and scaled source must be same size.\n");

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  int nt = m->get_axis(4).n;
  int ny = m->get_axis(3).n;
  int nx = m->get_axis(2).n;
  int nz = m->get_axis(1).n;

  for(int it = 0; it < nt; ++it) {
    for(int iy = 0; iy < ny; ++iy) {
      for(int ix = 0; ix < nx; ++ix) {
        for(int iz = 0; iz < nz; ++iz) {
          int uidx = it*ny*nx*nz + iy*nx*nz + ix*nz + iz;
          int vidx =               iy*nx*nz + ix*nz + iz;
          m->vals[uidx] += -_v2dtu2->vals[vidx] * d->vals[uidx];
        }
      }
    }
  }
  return true;
}

