/*
 * taper.cpp
 *
 *  Created on: Jan 4, 2017
 *      Author: joe
 */

#include <math.h>
#include "tapercos.h"

tapercos::tapercos(std::vector<axis> azx, int bz, int bx, int ord, float alpha) {
  _nz = azx[0].n; _nx = azx[1].n;
  _bz = bz;       _bx = bx;
  _alpha = alpha;
  _tap = new hypercube_float(azx, true);
  _shift = ord/2;
  _nshift = (_nx-_bx-_shift)*(_nz);
  build_taper(_tap);
}

void tapercos::build_taper(hypercube_float *in) {

  /* Left and Right */
  for(int ix = _shift; ix < _bx+_shift; ++ix) {
    for(int iz = _bz+_shift; iz < _nz-_bz-_shift; ++iz) {
      double tap = 1.0 * (_bx - (ix - _shift))/(_bx);
      tap = _alpha + (1.0 - _alpha) * cos(M_PI*tap);
      _tap->vals[ix        *_nz + iz] = tap;
      _tap->vals[(_nx-ix-1)*_nz + iz] = tap;
    }
  }

  /* Top and Bottom */
  for(int iz = _shift; iz < _bz+_shift; ++iz) {
    for(int ix = _bx+_shift; ix < _nx-_bx-_shift; ++ix) {
      double tap = 1.0 * (_bz - (iz - _shift))/(_bz);
      tap = _alpha + (1.0 - _alpha) * cos(M_PI*tap);
      _tap->vals[ix*_nz + iz      ] = tap;
      _tap->vals[ix*_nz + _nz-iz-1] = tap;
    }
  }

  /* Corners */
  for(int ix = _shift; ix < _bx+_shift; ++ix) {
    for(int iz = _shift; iz < _bz+_shift; ++iz) {
      double tap = 1.0*(_bx - (ix - _shift))*(_bx - (ix - _shift))/(_bx*_bx) * 1.0*(_bz - (iz - _shift))*(_bz - (iz - _shift))/(_bz*_bz);
      tap = sqrt(tap);
      tap = _alpha + (1.0 - _alpha)*cos(M_PI*tap);
      _tap->vals[ix        *_nz + iz      ] = tap;
      _tap->vals[(_nx-ix-1)*_nz + iz      ] = tap;
      _tap->vals[ix        *_nz + _nz-iz-1] = tap;
      _tap->vals[(_nx-ix-1)*_nz + _nz-iz-1] = tap;
    }
  }

  /* Middle */
  for(int ix = _bx + _shift; ix < _nx-_bx-_shift; ++ix) {
    for(int iz = _bz + _shift; iz < _nz-_bz-_shift; ++iz)  {
      _tap->vals[ix*_nz + iz] = 1.0;
    }
  }
}

void tapercos::apply(hypercube_float *cur, hypercube_float *nex) {

  /* Left and Right application */
  for(int ileft = _shift; ileft < (_bx+_shift)*(_nz); ++ileft) {
    int irite = _nshift + ileft;
    cur->vals[ileft] *= _tap->vals[ileft];
    nex->vals[ileft] *= _tap->vals[ileft];
    cur->vals[irite] *= _tap->vals[irite];
    nex->vals[irite] *= _tap->vals[irite];
  }

  /* Top and Bottom application */
  for(int ix = _bx+_shift; ix < _nx-_bx-_shift; ++ix) {
    for(int iz = _shift; iz < _bz+_shift; ++iz) {
      cur->vals[ix*_nz +       iz] *= _tap->vals[ix*_nz +       iz];
      nex->vals[ix*_nz +       iz] *= _tap->vals[ix*_nz +       iz];
      cur->vals[ix*_nz + _nz-iz-1] *= _tap->vals[ix*_nz + _nz-iz-1];
      nex->vals[ix*_nz + _nz-iz-1] *= _tap->vals[ix*_nz + _nz-iz-1];
    }
  }
}

hypercube_float* tapercos::get_taper() {
  return _tap;
}
