extern "C" {
#include "seplib.h"
}
#include <complex>
#include <random>
#include "wavelet.h"

wavelet::wavelet(axis st) {
  _ntsrc = st.n;
  _dtsrc = st.d;
  _ftsrc = st.o;
}

wavelet::wavelet(axis st, float dly) {
  _ntsrc = st.n;
  _dtsrc = st.d;
  _ftsrc = st.o;
  _dly   = dly;
}

void wavelet::ricker(float f, float amp, hypercube_float *src) {
  for (int it = 0;  it < _ntsrc; ++it) {
    float t = it * _dtsrc - _dly;
    float pift = M_PI * M_PI * (f * f) * (t * t);
    src->vals[it] = amp*((1 - 2 * pift) * exp(-pift));
  }
}

void wavelet::monosin(float f, float amp, float ph, hypercube_float *src) {
  float phrad = ph*DEG2RAD;
  for(int it = 0; it < _ntsrc; ++it) {
    float t = it * _dtsrc;
    src->vals[it] = amp*cos(f * 2*M_PI* t - phrad);
  }
}

void wavelet::sumsin(std::vector<float> fs, std::vector<float> amps, std::vector<float>phs, hypercube_float* src) {
  int nsig = fs.size();
  src->zero(); std::vector<float> phrads;
  /* Convert to radians */
  for(int is = 0; is < nsig; ++is) {phrads.push_back(phs[is]*DEG2RAD);}
  if(nsig != (int)amps.size() && nsig != (int)phs.size()) seperr("Num freqs, amps and phzs must be the same\n.");
  for(int it = 0; it < _ntsrc; ++it) {
    for(int is = 0; is < nsig; ++is) {
      float t = it * _dtsrc;
      src->vals[it] += amps[is]*cos(fs[is]* 2*M_PI * t - phrads[is]);
    }
  }
}

void wavelet::morlet(float f, float amp, float ph, float c, int iramp, hypercube_float *src) {
  float pi4 = 1/(pow(M_PI,4));
  float scale = 1/(2*c*c);
  float phrad = ph*DEG2RAD;
  for(int it = 0; it < _ntsrc; ++it) {
    float t = it * _dtsrc - _dly;
    float sinu = amp*cos(f * 2*M_PI* t - phrad);
    if(iramp == 1) {
      src->vals[it] = it*pi4*sinu*exp(-t*t*scale);
    }
    else if(iramp == 2) {
      src->vals[it] = (_ntsrc-it)*pi4*sinu*exp(-t*t*scale);
    }
    else {
      src->vals[it] = pi4*sinu*exp(-t*t*scale);
    }
  }
}
