/*
 * wfldlint.cpp
 *
 *  Created on: Nov 24, 2016
 *      Author: joe
 */

#include "wfldlint.h"
#include <math.h>

bool wfldlint::forward(bool add, my_vector* mod, my_vector* dat) {
  if(!add && dat) dat->zero();

  hypercube_float *m = (hypercube_float*) mod;
  hypercube_float *d = (hypercube_float*) dat;

  if(m->get_ndim() != d->get_ndim() && d->get_ndim()!= 3)
    seperr("Wfldlint: This interpolation is designed for 3D hypercubes."
        " Please check inputs\n.");

  if(m->get_axis(1).n != d->get_axis(1).n)
    seperr("Wfldlint: nz from model and data must be same.\n");

  if(m->get_axis(2).n != d->get_axis(2).n)
    seperr("Wfldlint: nx from model and data must be same.\n");

  int nz = d->get_axis(1).n;
  int nx = d->get_axis(2).n;
  int nt = d->get_axis(3).n;

  int offset = nx*nz;
  for(int ix = 0; ix < nx; ++ix) {
    for(int iz = 0; iz < nz; ++iz) {
      for(int it = 0; it < nt; ++it) {
        int idx = it*nz*nx + ix*nz + iz;
        double dcoord = d->get_axis(3).o + d->get_axis(3).d * static_cast<double>((idx/(offset)));
        double f = (dcoord - m->get_axis(3).o)/m->get_axis(3).d;
        int im = floor(f);
        double fx = f - im; double gx = 1 - fx;
        im = im*nz*nx + ix*nz + iz;
        if(im >= 0 && im < m->get_n123() - offset) {
          d->vals[idx] += gx * m->vals[im] + fx * m->vals[im+offset];
        }
      }
    }
  }
  return true;
}

bool wfldlint::adjoint(bool add, my_vector* mod, my_vector* dat) {
  if(!add && mod) mod->zero();

  hypercube_float *m = (hypercube_float*) mod;
  hypercube_float *d = (hypercube_float*) dat;

  if(m->get_ndim() != d->get_ndim() && d->get_ndim()!= 3)
    seperr("Wfldlint: This interpolation is designed for 3D hypercubes."
        " Please check inputs\n.");

  if(m->get_axis(1).n != d->get_axis(1).n)
    seperr("Wfldlint: nz from model and data must be same.\n");

  if(m->get_axis(2).n != d->get_axis(2).n)
    seperr("Wfldlint: nx from model and data must be same.\n");

  int nz = d->get_axis(1).n;
  int nx = d->get_axis(2).n;
  int nt = d->get_axis(3).n;

  int offset = nx * nz;
  for(int ix = 0; ix < nx; ++ix) {
    for(int iz = 0; iz < nz; ++iz) {
      for(int it = 0; it < nt; ++it) {
        int idx = it*nz*nx + ix*nz + iz;
        double dcoord = d->get_axis(3).o + d->get_axis(3).d * static_cast<double>((idx/(offset)));
        double f = (dcoord - m->get_axis(3).o)/m->get_axis(3).d;
        int im = floor(f);
        double fx = f - im; double gx = 1 - fx;
        im = im*nz*nx + ix*nz + iz;
        if(im >= 0 && im < m->get_n123() - offset) {
          m->vals[im       ] += gx * d->vals[idx];
          m->vals[im+offset] += fx * d->vals[idx];
        }
      }
    }
  }
  return true;
}
