/*
 * wfldlint3d.cpp
 *
 *  Created on: Jan 8, 2017
 *      Author: joe
 */

#include "wfldlint3d.h"

wfldlint3d::wfldlint3d(std::vector<axis> saxis, std::vector<axis> uaxis) {
  /* Axes */
  _nts = saxis[3].n;
  _sample = saxis[3].d/uaxis[3].d;
  _onestp = saxis[0].n * saxis[1].n * saxis[2].n;
  _slaxis.push_back(saxis[0]); _slaxis.push_back(saxis[1]);
  _slaxis.push_back(saxis[2]);
  /* Compute interpolation coefficients */
  _ls1 = interp_coef(_sample).first;
  _ls2 = interp_coef(_sample).second;
  /* Allocate temporary cubes */
  _slc1 = new hypercube_float(_slaxis, true);
  _slc2 = new hypercube_float(_slaxis, true);
  _tmp  = new hypercube_float(_slaxis, true);
}

std::pair<std::vector<double>,std::vector<double>> wfldlint3d::interp_coef(int sample) {
  std::vector<double> ls1, ls2;
  for(int is = 0; is < sample; ++is) {
    ls1.push_back(static_cast<double>((sample-is))/static_cast<double>(sample));
    ls2.push_back(1.0-ls1[is]);
    ls1[is] /= sample; ls2[is] /= sample;
  }
  return std::pair<std::vector<double>,std::vector<double>>(ls1,ls2);
}

bool wfldlint3d::forward(bool add, my_vector *mod, my_vector *dat) {
  if(!add && dat) dat->zero();

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  _slc1->zero(); _slc2->zero();

  int k = 0;
  for(int it = 0; it < _nts-1; ++it) {
    /* Extract cubes */
    memcpy(_slc1->vals,&m->vals[(it+0)*_onestp],sizeof(float)*_onestp);
    memcpy(_slc2->vals,&m->vals[(it+1)*_onestp],sizeof(float)*_onestp);
    for(int is = 0; is < _sample; ++is) {
      _tmp->scale_add(0.0,_slc1,_ls1[is]);
      _tmp->scale_add(1.0,_slc2,_ls2[is]);
      memcpy(&d->vals[k*(_onestp)],_tmp->vals,sizeof(float)*_onestp);
      ++k;
    }
  }
  return true;
}

bool wfldlint3d::adjoint(bool add, my_vector *mod, my_vector *dat) {
  if(!add && mod) mod->zero();

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  _slc1->zero(); _slc2->zero();

  int k = 0;
  for(int it = 0; it < _nts-1; ++it) {
    /* Extract cubes */
    for(int is = 0; is < _sample; ++is) {
      memcpy(_tmp->vals,&d->vals[k*_onestp],sizeof(float)*_onestp);
      _slc1->scale_add(1.0,_tmp,_ls1[is]);
      _slc2->scale_add(1.0,_tmp,_ls2[is]);
      ++k;
    }
    memcpy(&m->vals[(it+0)*_onestp],_slc1->vals,sizeof(float)*_onestp);
    memcpy(&m->vals[(it+1)*_onestp],_slc2->vals,sizeof(float)*_onestp);
    _slc1->set(_slc2->vals);
    _slc2->zero();
  }
  return true;
}
