/**
 * Linear interpolation operator
 * intended for interpolating the temporal
 * axis of 3D wavefields.
 * @author: Joseph Jennings
 * @version: 2017.01.08
 */
#ifndef WFLDLINT3D_H_
#define WFLDLINT3D_H_

#include <vector>
#include "my_operator.h"
#include "hypercube_float.h"

class wfldlint3d : public my_operator {
  public:
    wfldlint3d(){};
    wfldlint3d(std::vector<axis> saxis, std::vector<axis> uaxis);
    bool forward(bool add, my_vector *mod, my_vector *dat);
    bool adjoint(bool add, my_vector *mod, my_vector *dat);
    ~wfldlint3d(){
      delete _slc1; delete _slc2; delete _tmp;
    };

  private:
    int _sample, _nts, _onestp;
    std::vector<axis> _slaxis;
    std::vector<double> _ls1, _ls2;
    std::pair<std::vector<double>,std::vector<double>>interp_coef(int sample);
    hypercube_float *_slc1, *_slc2, *_tmp;
};

#endif /* WFLDLINT3D_H_ */
