/*
 * zeropad.cpp
 *
 *  Created on: Oct 28, 2016
 *      Author: joe
 */

#include "zeropad.h"
#include "hypercube_float.h"

zeropad::zeropad(int xloc, int zloc) {
  _xloc = xloc; _zloc = zloc;
}

zeropad::zeropad(int xloc, int zloc, float val) {
  _xloc = xloc; _zloc = zloc;
  _val = val;
}

bool zeropad::forward(bool add, my_vector *mod, my_vector *dat) {
  if(!add && dat) dat->zero();
  hypercube_float *m  = (hypercube_float*)mod;
  hypercube_float *d  = (hypercube_float*)dat;

  if(m->get_ndim() != 1)
    seperr("Zero Pad: Input source wavelet must be 1D.\n");
  if(m->get_axis(1).n != d->get_axis(3).n)
    seperr("Zero Pad: Time axes are not same length. \n");

  int nx = d->get_axis(2).n; int nz = d->get_axis(1).n;
  for(int it = 0; it < m->get_axis(1).n; ++it) {
    int index = it * nx * nz + _xloc * nz + _zloc;
    d->vals[index] += m->vals[it];
  }

  return true;
}

bool zeropad::adjoint(bool add, my_vector *mod, my_vector *dat) {
  if(!add && mod) mod->zero();
  hypercube_float *m  = (hypercube_float*)mod;
  hypercube_float *d  = (hypercube_float*)dat;

  if(m->get_ndim() != 1)
    seperr("Zero Pad: Input source wavelet must be 1D.\n");
  if(m->get_axis(1).n != d->get_axis(3).n)
    seperr("Zero Pad: Time axes are not same length. \n");

  int nx = d->get_axis(2).n; int nz = d->get_axis(1).n;
  for(int it = 0; it < m->get_axis(1).n; ++it) {
    int index = it * nx * nz + _xloc * nx + _zloc;
    m->vals[it] += d->vals[index];
  }

  return true;
}
