/*
 * zeropad3d.cpp
 *
 *  Created on: Jan 7, 2017
 *      Author: joe
 */

#include "zeropad3d.h"
#include "hypercube_float.h"

zeropad3d::zeropad3d(int zloc, int xloc, int yloc) {
  _zloc = zloc; _xloc = xloc; _yloc = yloc;
}

zeropad3d::zeropad3d(int zloc, int xloc, int yloc, float val) {
  _zloc = zloc; _xloc = xloc; _yloc = yloc;
  _val = val;
}

bool zeropad3d::forward(bool add, my_vector *mod, my_vector *dat) {
  if(!add && dat) dat->zero();

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  if(m->get_ndim() != 1)
    seperr("Zero Pad: Input source wavelet must be 1D.\n");
  if(m->get_axis(1).n != d->get_axis(4).n)
    seperr("Zero Pad: Time axes are not same length.\n");

  /* Spatial lengths */
  int nx = d->get_axis(2).n; int nz = d->get_axis(1).n;
  int ny = d->get_axis(3).n;
  for(int it = 0; it < m->get_axis(1).n; ++it) {
    int idx = it*ny*nx*nz + _yloc*nx*nz + _xloc*nz + _zloc;
    d->vals[idx] += m->vals[it];
  }
  return true;
}

bool zeropad3d::adjoint(bool add, my_vector *mod, my_vector *dat) {
  if(!add && mod) mod->zero();

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  if(m->get_ndim() != 1)
    seperr("Zero Pad: Input source wavelet must be 1D.\n");
  if(m->get_axis(1).n != d->get_axis(4).n)
    seperr("Zero Pad: Time axes are not same length.\n");

  /* Spatial lengths */
  int nx = d->get_axis(2).n; int nz = d->get_axis(1).n;
  int ny = d->get_axis(3).n;
  for(int it = 0; it < m->get_axis(1).n; ++it) {
    int idx = it*ny*nx*nz + _yloc*nx*nz + _xloc*nz + _zloc;
    m->vals[it] += d->vals[idx];
  }
  return true;
}

