/*
 * zeropadcub.cpp
 *
 *  Created on: Jan 7, 2017
 *      Author: joe
 */

#include "zeropadcub.h"
#include "hypercube_float.h"

zeropadcub::zeropadcub(int zloc, int xloc, int yloc) {
  _zloc = zloc; _xloc = xloc; _yloc = yloc;
}

zeropadcub::zeropadcub(int zloc, int xloc, int yloc, float fill) {
  _zloc = zloc; _xloc = xloc; _yloc = yloc;
  _fill = fill;
}

bool zeropadcub::forward(bool add, my_vector *mod, my_vector *dat) {
  dat->set_val(_fill);

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  /* Get spatial lengths */
  int nzp = d->get_axis(1).n; int nxp = d->get_axis(2).n;
  int nyp = d->get_axis(3).n;
  int nz  = m->get_axis(1).n; int nx  = m->get_axis(2).n;
  int ny  = m->get_axis(3).n;

  if(nzp < nz || nxp < nx || nyp < ny) {
    seperr("Output padded cube will be smaller than input. Please try again with n*p > n*.\n");
  }
  if(_xloc >= nx || _zloc >= nz || _yloc >= ny) {
    seperr("Initial location is out of bounds. Please try again with *loc < n*.\n");
  }

  for(int iy = 0; iy < ny; ++iy) {
    for(int ix = 0; ix < nx; ++ix) {
      for(int iz = 0; iz < nz; ++iz) {
        d->vals[(iy+_yloc)*nxp*nzp + (ix+_xloc)*nzp + iz + _zloc] = m->vals[iy*nx*nz + ix*nz + iz];
      }
    }
  }
  return true;
}

bool zeropadcub::adjoint(bool add, my_vector *mod, my_vector *dat) {

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  /* Get spatial lengths */
  int nzp = d->get_axis(1).n; int nxp = d->get_axis(2).n;
  int nyp = d->get_axis(3).n;
  int nz  = m->get_axis(1).n; int nx  = m->get_axis(2).n;
  int ny  = m->get_axis(3).n;

  if(nzp < nz || nxp < nx || nyp < ny) {
    seperr("Output padded cube will be smaller than input. Please try again with n*p > n*.\n");
  }
  if(_xloc >= nx || _zloc >= nz || _yloc >= ny) {
    seperr("Initial location is out of bounds. Please try again with *loc < n*.\n");
  }

  for(int iy = 0; iy < ny; ++iy) {
    for(int ix = 0; ix < nx; ++ix) {
      for(int iz = 0; iz < nz; ++iz) {
        m->vals[iy*nx*nz + ix*nz + iz] = d->vals[(iy+_yloc)*nxp*nzp + (ix+_xloc)*nzp + iz + _zloc];
      }
    }
  }
  return true;
}
