
#include "zeropadimg.h"
#include "hypercube_float.h"

zeropadimg::zeropadimg(int zloc, int xloc, float fill) {
  _zloc = zloc; _xloc = xloc; _fill = fill;
}

bool zeropadimg::forward(bool add, my_vector *mod, my_vector *dat) {
  dat->set_val(_fill);

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  int nzp = d->get_axis(1).n; int nxp = d->get_axis(2).n;
  int nz  = m->get_axis(1).n; int nx  = m->get_axis(2).n;

  //FIXME: Probably should return false instead.
  if(nzp < nz || nxp < nx) {
    seperr("Output padded image will be smaller than input. Please try again with nzp > nz.\n");
  }
  if(_xloc >= nx || _zloc >= nz) {
    seperr("Initial location is out of bounds. Please try again with xloc < nx or zloc < nz.\n");
  }

  for(int ix = 0; ix < nx; ++ix) {
    for(int iz = 0; iz < nz; ++iz) {
      d->vals[(ix+_xloc)*nzp + iz + _zloc] = m->vals[ix*nz + iz];
    }
  }
  return true;
}

bool zeropadimg::adjoint(bool add, my_vector *mod, my_vector *dat) {

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  int nzp = d->get_axis(1).n; int nxp = d->get_axis(2).n;
  int nz  = m->get_axis(1).n; int nx  = m->get_axis(2).n;

  if(nzp < nz || nxp < nx) {
    seperr("Output padded image will be smaller than input. Please try again with nzp > nz.\n");
  }
  if(_xloc >= nx || _zloc >= nz) {
    seperr("Initial location is out of bounds. Please try again with xloc < nx or zloc < nz.\n");
  }

   for(int ix = 0; ix < nx; ++ix) {
     for(int iz = 0; iz < nz; ++iz) {
       m->vals[ix*nz + iz] = d->vals[(ix+_xloc)*nzp + iz+_zloc];
     }
   }
  return true;
}
