#include "zeropadwfld.h"
#include "hypercube_float.h"

zeropadwfld::zeropadwfld(int zloc, int xloc) {
  _zloc = zloc; _xloc = xloc; _fill = 0.0;
}

zeropadwfld::zeropadwfld(int zloc, int xloc, float fill) {
  _zloc = zloc; _xloc = xloc; _fill = fill;
}

bool zeropadwfld::forward(bool add, my_vector *mod, my_vector *dat) {
  dat->set_val(_fill);

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  int nt  = m->get_axis(3).n;
  int nzp = d->get_axis(1).n; int nxp = d->get_axis(2).n;
  int nz  = m->get_axis(1).n; int nx  = m->get_axis(2).n;

  if(nt != d->get_axis(3).n) {
    seperr("Time axes must be the same for both wavefields\n.");
  }
  if(nzp < nz || nxp < nx) {
    seperr("Output padded image will be smaller than input. Please try again with nzp > nz.\n");
  }
  if(_xloc >= nx || _zloc >= nz) {
    seperr("Initial location is out of bounds. Please try again with xloc < nx or zloc < nz.\n");
  }

  for(int it = 0; it < nt; ++it) {
    for(int ix = 0; ix < nx; ++ix) {
      for(int iz = 0; iz < nz; ++iz) {
        d->vals[it*nxp*nzp + (ix+_xloc)*nzp + iz + _zloc] = m->vals[it*nz*nz + ix*nz + iz];
      }
    }
  }
  return true;
}

bool zeropadwfld::adjoint(bool add, my_vector *mod, my_vector *dat) {
  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  int nt  = d->get_axis(3).n;
  int nzp = d->get_axis(1).n; int nxp = d->get_axis(2).n;
  int nz  = m->get_axis(1).n; int nx  = m->get_axis(2).n;

  if(nt != m->get_axis(3).n) {
    seperr("Time axes must be the same for both wavefields\n.");
  }
  if(nzp < nz || nxp < nx) {
    seperr("Output padded image will be smaller than input. Please try again with nzp > nz.\n");
  }
  if(_xloc >= nx || _zloc >= nz) {
    seperr("Initial location is out of bounds. Please try again with xloc < nx or zloc < nz.\n");
  }

  for(int it = 0; it < nt; ++it) {
    for(int ix = 0; ix < nx; ++ix) {
      for(int iz = 0; iz < nz; ++iz) {
        m->vals[it*nx*nz + ix*nz + iz] = d->vals[it*nxp*nzp + (ix+_xloc)*nzp + iz + _zloc];
      }
    }
  }
  return true;
}
