/**
 * A simple program to test
 * 3D acoustic, constant density
 * wave propagation
 * @author: Joseph Jennings
 * @version: 2017.01.09
 */

#include "sep_params.h"
#include "../main/afd3d.h"
#include "../main/laplacian3d4th.h"
#include "../main/laplacian3d10th.h"

int main(int argc, char **argv) {

  sep_params pars = sep_params(argc, argv);

  int adj = pars.get_int("adj",0);
  int ord = pars.get_int("ord",4);
  float fmax = pars.get_float("fmax",50.0);
  std::string mode   = pars.get_string("mode","slow");
  int bz = 0; int bx = 0; float alpha = 0.0;
  int ntu = 0; float otu = 0.0; float dtu = 0.0;

  sregf srcin; sregf wfdin;
  if(adj) {
    wfdin = sregf("in");
  } else {
    srcin = sregf("in");
  }
  sregf velin = sregf("vel");
  hypercube_float *src = nullptr; hypercube_float *wfd = nullptr;
  hypercube_float *vel = new hypercube_float(velin.get_axes(), true);

  if(adj) {
  } else {
    src = new hypercube_float(srcin.get_axes(), true);
    std::vector<axis> uaxis;
    uaxis.push_back(vel->get_axis(1)); uaxis.push_back(vel->get_axis(2));
    uaxis.push_back(vel->get_axis(3)); uaxis.push_back(srcin.get_axis(4));
    wfd = new hypercube_float(uaxis, true);
  }

  velin.read_all("vel",vel);

  laplacian3d* lap = nullptr;
  if(ord == 4) {
    lap = new laplacian3d4th(wfd->get_axes());
  } else if(ord == 10) {
    lap = new laplacian3d10th(wfd->get_axes());
  } else {
    seperr("Please either choose 4th or 10th order. Order %d has not yet been implemented.\n",ord);
  }

  afd3d *prp = new afd3d(wfd->get_axes(), vel, lap, fmax);

  if(adj) {
  } else {
    srcin.read_all("in",src);
    prp->forward(false,src,wfd);
    sregf wout = sregf(wfd->get_axes());
    wout.write_description("out");
    wout.write_all("out",wfd);
  }

  delete vel; delete wfd; delete src;
}
