/**
 * A program to test the wave propagation
 * boundary derivative operator
 * @author: Joseph Jennings
 * @version: 2016.11.27
 */

#include "../main/bcder.h"
#include "hypercube_float.h"
#include "sregf.h"
#include "sep_params.h"

int main(int argc, char **argv) {

  sep_params pars = sep_params(argc, argv);

  int ord = pars.get_int("ord",10);
  sregf fin = sregf("in");

  std::vector<int>nbs; nbs.resize(4);

  /* Get command line parameters */
  int adj = pars.get_int("adj",0);
  int slc = pars.get_int("slc",10);
  nbs[0]  = pars.get_int("bz1",0);
  nbs[2]  = pars.get_int("bz2",fin.get_axis(1).n);
  nbs[1]  = pars.get_int("bx1",0);
  nbs[3]  = pars.get_int("bx2",fin.get_axis(2).n);

  /* Temporary 2D array */
  std::vector<axis> ax2d;
  ax2d.push_back(fin.get_axis(1));
  ax2d.push_back(fin.get_axis(2));
  hypercube_float *tmp2d= new hypercube_float(ax2d,true);
  int nz = fin.get_axis(1).n; int nx = fin.get_axis(2).n;
  /* For writing 2D array */
  std::vector<axis> ax3d0;
  ax3d0 = ax2d; ax3d0.push_back(axis(1,0.0,0.0));

  hypercube_float *mod = new hypercube_float(ax2d,true);
  hypercube_float *dat = new hypercube_float(ax2d,true);

  bcder bdop = bcder(dat->get_axes(),nbs,ord);

  if(adj) {
    fin.read_all("in",dat);
    bdop.adjoint(false,mod,dat);
    sregf fout = sregf(mod->get_axes());
    fout.write_description("out");
    fout.write_all("out",mod);
  } else {
    fin.read_all("in",mod);
    bdop.forward(false,mod,dat);
    sregf fout = sregf(dat->get_axes());
    fout.write_description("out");
    fout.write_all("out",dat);
  }

  delete mod; delete dat;
}
