/**
 * A simple program to test the
 * Born scattering operator
 * @author: Joseph Jennings
 * @version: 2016.12.28
 */

#include "sep_params.h"
#include "sregf.h"
#include "hypercube_float.h"
#include "../main/bscat.h"

int main(int argc, char **argv) {

  sep_params pars = sep_params(argc, argv);

  int adj = pars.get_int("adj",0);

  /* Background wavefield */
  sregf u0in = sregf("u0");
  hypercube_float *u0 = new hypercube_float(u0in.get_axes(), true);
  u0in.read_all("u0",u0);

  /* Scattering operator */
  bscat brn = bscat(u0);

  /* Model and data hypercubes */
  hypercube_float *dm, *swfd;
  if(adj) {
    sregf swin = sregf("in");
    swfd = new hypercube_float(swin.get_axes(), true);
    swin.read_all("in",swfd);
    std::vector<axis> dmaxes;
    dmaxes.push_back(swin.get_axis(1));
    dmaxes.push_back(swin.get_axis(2));
    dmaxes.push_back(axis(1,0.0,1.0));
    dm = new hypercube_float(dmaxes, true);
    brn.adjoint(false,dm,swfd);
    sregf dmout = sregf(dm->get_axes());
    dmout.write_description("out");
    dmout.write_all("out",dm);
  } else {
    sregf dmin = sregf("in");
    dm = new hypercube_float(dmin.get_axes(), true);
    dmin.read_all("in",dm);
    swfd = new hypercube_float(u0->get_axes(), true);
    brn.forward(false,dm,swfd);
    sregf swout = sregf(swfd->get_axes());
    swout.write_description("out");
    swout.write_all("out",swfd);
  }

  delete dm; delete swfd;
}
