/**
 * A program to test the it-1
 * FD coefficient operator
 * @author: Joseph Jennings
 * @version: 2016.11.27
 */

#include "sep_params.h"
#include "../main/laplacian10th.h"
#include "../main/laplacian4th.h"
#include "../main/fdcoefb.h"
#include "hypercube_float.h"
#include "../main/simpleplot.h"

int main(int argc, char **argv) {

  sep_params pars = sep_params(argc, argv);

  int ord = pars.get_int("ord",10);
  float dtu2 = 0.001 * 0.001;

  sregf fin  = sregf("in");

  std::vector<int>nbs; nbs.resize(4);

  /* Get command line parameters */
  int adj = pars.get_int("adj",0);
  nbs[0]  = pars.get_int("bz1",0);
  nbs[2]  = pars.get_int("bz2",fin.get_axis(1).n);
  nbs[1]  = pars.get_int("bx1",0);
  nbs[3]  = pars.get_int("bx2",fin.get_axis(2).n);

  hypercube_float *dat   = new hypercube_float(fin.get_axes(),  true);
  hypercube_float *vel   = new hypercube_float(fin.get_axes(),  true);
  hypercube_float *wgt   = new hypercube_float(fin.get_axes(),  true);
  hypercube_float *vwt   = new hypercube_float(fin.get_axes(), true);
  hypercube_float *mod   = new hypercube_float(fin.get_axes(),  true);

  vel->set_val(3000.0); wgt->set_val(0.01); vwt->set_val(0.1);

  vel->mult(vel); vel->scale(dtu2);

  laplacian *lapop;
  if(ord == 10) {
    lapop = new laplacian10th(fin.get_axes());
  } else if(ord == 4) {
    lapop = new laplacian4th(fin.get_axes());
  } else {
    seperr("Only 4th and 10th order laplacians implemented so far\n.");
  }

  fdcoefb fdop = fdcoefb(fin.get_axes(),lapop,vel,wgt,vwt,nbs);

  if(adj) {
  } else {
    fin.read_all("in",mod);
    fdop.forward(false,mod,dat);
    sregf fout = sregf(dat->get_axes());
    fout.write_description("out");
    fout.write_all("out",dat);
  }

  delete vel; delete wgt; delete vwt;
  delete mod; delete dat;
}
