/**
 * Tests the laplacian operator
 * on a small 2D image from GEE
 * @author: Joseph Jennings
 * @version: 2016.11.25
 */

#include "sep_params.h"
#include "../main/laplacian10th.h"
#include "../main/laplacian4th.h"
#include "../main/simpleplot.h"

int main(int argc, char **argv) {

  sep_params pars = sep_params(argc, argv);

  int adj = pars.get_int("adj",0);
  int slc = pars.get_int("slc",10);
  int ord = pars.get_int("ord",4);

  sregf fin = sregf("in");

  hypercube_float *dat; hypercube_float *mod;

  /* Temporary 2D array */
  std::vector<axis> ax2d;
  ax2d.push_back(fin.get_axis(1));
  ax2d.push_back(fin.get_axis(2));
  hypercube_float *tmp2d= new hypercube_float(ax2d,true);
  int nz = fin.get_axis(1).n; int nx = fin.get_axis(2).n;
  /* For writing 2D array */
  std::vector<axis> ax3d0;
  ax3d0 = ax2d; ax3d0.push_back(axis(1,0.0,0.0));

  /* Build hypercubes */
  if(fin.get_ndim() == 3) {
    if(adj) {
      dat = new hypercube_float(fin.get_axes(),true);
      mod = new hypercube_float(ax2d,true);
    } else {
      mod = new hypercube_float(fin.get_axes(),true);
      dat = new hypercube_float(ax2d,true);
    }
  }
  else if(fin.get_ndim() == 2) {
    mod = new hypercube_float(ax2d,true);
    dat = new hypercube_float(ax2d,true);
  }
  else {
    seperr("Program is not implemented for ndim != 2 || ndim != 3.\n");
  }
  /* Laplacian operator */
  laplacian *lap;
  if(ord == 4) {
    lap = new laplacian4th(dat->get_axes());
  } else if(ord == 10) {
    lap = new laplacian10th(dat->get_axes());
  } else {
    seperr("Only 4th and 10th order laplacian implemented for now.\n");
  }

  if(adj) {
    fin.read_all("in",dat);
    if(fin.get_ndim() == 2) {
      lap->adjoint(false,mod,dat);
    }
    else {
      /* Copy slice to 2D array */
      memcpy(tmp2d->vals,&dat->vals[slc*nx*nz],sizeof(float)*nx*nz);
      lap->adjoint(false,mod,tmp2d);
    }
    sregf fout = sregf(mod->get_axes());
    fout.write_description("out");
    fout.write_all("out",mod);
  } else {
    fin.read_all("in",mod);
    if(fin.get_ndim() == 2) {
      lap->forward(false,mod,dat);
    }
    else {
      /* Copy slice to 2D array */
      memcpy(tmp2d->vals,&mod->vals[slc*nx*nz],sizeof(float)*nx*nz);
      lap->forward(false,tmp2d,dat);
      /* Test */
      hypercube_float *tmp3d = new hypercube_float(mod->get_axes(),true);
      memcpy(&tmp3d->vals[slc*nx*nz],&mod->vals[slc*nx*nz],sizeof(float)*nx*nz);
      simpleplot::plot_slice2d3(tmp3d,slc);
      tmp2d->zero();
      memcpy(tmp2d->vals,tmp3d->vals,sizeof(float)*nx*nz);
      memcpy(tmp2d->vals,&tmp3d->vals[slc*nx*nz],sizeof(float)*nx*nz);
      simpleplot::plot2d(tmp2d);
    }
    sregf fout = sregf(ax3d0);
    fout.write_description("out");
    fout.write_all("out",dat);
  }

  delete mod; delete dat; delete tmp2d;
  delete lap;
}
