/**
 * A simple program to test
 * the 3D laplacian operator
 * @author: Joseph Jennings
 * @version: 2017.01.08
 */

#include "sep_params.h"
#include "../main/laplacian3d.h"
#include "../main/laplacian3d4th.h"
#include "../main/laplacian3d10th.h"

int main(int argc, char **argv) {

  sep_params pars = sep_params(argc, argv);

  int adj = pars.get_int("adj",0);
  int ord = pars.get_int("ord",4);

  sregf fin = sregf("in");

  /* Build hypercubes */
  hypercube_float *dat = new hypercube_float(fin.get_axes(), true);
  hypercube_float *mod = new hypercube_float(fin.get_axes(), true);

  /* 3D laplacian operator */
  laplacian3d* lap3d = nullptr;
  if(ord == 4) {
    lap3d = new laplacian3d4th(dat->get_axes());
  } else if(ord == 10) {
    lap3d = new laplacian3d10th(dat->get_axes());
  } else {
    seperr("Only the 4th and 10th order 3D laplacian implemented for now.\n");
  }

  if(adj) {
    fin.read_all("in",dat);
    lap3d->adjoint(false,mod,dat);
    fin.write_description("out");
    fin.write_all("out",mod);
  } else {
    fin.read_all("in",mod);
    lap3d->forward(false,mod,dat);
    fin.write_description("out");
    fin.write_all("out",dat);
  }

  delete mod; delete dat; delete lap3d;
}
