/**
 * A simple program to test
 * the receiver restriction operator
 * @author: Joseph Jennings
 * @version: 2016.12.27
 */

#include "sep_params.h"
#include "axis.h"
#include "sregf.h"
#include "hypercube_float.h"
#include "../main/rrec.h"
#include "../main/rrec3d.h"

int main(int argc, char **argv) {

  sep_params pars = sep_params(argc, argv);

  int adj   = pars.get_int("adj",0);
  /* Data parameters */
  int nrx   = pars.get_int("nrx",200);
  float orx = pars.get_float("orx",51);
  float drx = pars.get_float("drx",1.0);
  int nry   = pars.get_int("nry",200);
  float ory = pars.get_float("ory",51);
  float dry = pars.get_float("dry",1.0);
  int recz  = pars.get_int("recz",150);
  /* Wavefield parameters */
  int   nz  = pars.get_float("nz",300);
  float oz  = pars.get_float("oz",0.0);
  float dz  = pars.get_float("dz",10.0);
  int   nx  = pars.get_int("nx",300);
  float ox  = pars.get_float("ox",0.0);
  float dx  = pars.get_float("dx",10.0);
  int   ny  = pars.get_int("nx",300);
  float oy  = pars.get_float("ox",0.0);
  float dy  = pars.get_float("dx",10.0);

  axis recx = axis(nrx,orx,drx);
  axis recy = axis(nry,ory,dry);

  /* Receiver restriction operator */
  rrec rr = rrec(recx,recz);

  hypercube_float *wfd = nullptr;
  hypercube_float *dat = nullptr;
  if(adj) {
    /* Input wavefield */
    sregf wfdin = sregf("in");
    wfd = new hypercube_float(wfdin.get_axes(), true);
    wfdin.read_all("in",wfd);
    /* Output data */
    std::vector<axis> dxs;
    if(wfd->get_ndim() == 3) {
      dxs.push_back(wfdin.get_axis(3)); dxs.push_back(recx);
      dxs.push_back(axis(1,0.0,0.0));
      dat = new hypercube_float(dxs, true);
      /* Receiver restriction */
      rr.adjoint(false,dat,wfd);
    } else if(wfd->get_ndim() == 4) {
      rrec3d rr3 = rrec3d(wfd->get_axes(),recx,recy,recz);
      dxs.push_back(wfdin.get_axis(4)); dxs.push_back(recx);
      dxs.push_back(recy); dxs.push_back(axis(1,0.0,0.0));
      dat = new hypercube_float(dxs, true);
      rr3.adjoint(false,dat,wfd);
    } else {
      seperr("rrec-test: Receiver restriction adjoint only operates on 2D or 3D wavefields.\n");
    }
    /* Write out data */
    sregf datout = sregf(dat->get_axes());
    datout.write_description("out");
    datout.write_all("out",dat);
  } else {
    /* Input data */
    sregf datin = sregf("in");
    dat = new hypercube_float(datin.get_axes(), true);
    datin.read_all("in",dat);
    /* Output wavefield */
    std::vector<axis> wfdax;
    if(dat->get_ndim() == 2) {
      wfdax.push_back(axis(nz,oz,dz));
      wfdax.push_back(axis(nx,ox,dx));
      wfdax.push_back(datin.get_axis(1));
      wfd = new hypercube_float(wfdax, true);
      rr.forward(false,dat,wfd);
    } else if(dat->get_ndim() == 3) {
      wfdax.push_back(axis(nz,oz,dz));
      wfdax.push_back(axis(nx,ox,dx));
      wfdax.push_back(axis(ny,oy,dy));
      wfdax.push_back(datin.get_axis(1));
      wfd = new hypercube_float(wfdax, true);
      rrec3d rr3 = rrec3d(wfd->get_axes(),recx,recy,recz);
      rr3.forward(false,dat,wfd);
    } else {
      seperr("rrec-test: Receiver restriction forward only operates on 2D or 3D data.\n");
    }
    /* Write out wavefield */
    sregf wfdout = sregf(wfd->get_axes());
    wfdout.write_description("out");
    wfdout.write_all("out",wfd);
  }

  /* Clean up */
  delete dat; delete wfd;
}
