/**
 * Simple program to test the
 * source scale operator
 * @author: Joseph Jennings
 * @version: 2016.11.24
 */

#include "../main/sourcescale.h"
#include "../main/sourcescale3d.h"
#include "../main/simpleplot.h"
#include "hypercube_float.h"
#include "sregf.h"
#include "sep_params.h"

int main(int argc, char **argv) {

  sep_params pars = sep_params(argc, argv);

  /* Get command line parameters */
  int adj   = pars.get_int("adj",0);
  float dtu = pars.get_float("dtu",0.001);
  std::string wname = pars.get_string("wgt","");

  sregf srcin = sregf("in");
  sregf velin = sregf("vel");
  sregf wgtin;

  hypercube_float *psrc = new hypercube_float(srcin.get_axes(), true);
  hypercube_float *ssrc = new hypercube_float(srcin.get_axes(), true);
  hypercube_float *vel  = new hypercube_float(velin.get_axes(), true);
  hypercube_float *wgt;

  velin.read_all("vel", vel);
  sourcescale *ss;
  sourcescale3d ss3 = sourcescale3d(dtu,vel);
  if(!wname.empty()) {
    wgtin = sregf("wgt");
    wgt = new hypercube_float(wgtin.get_axes(), true);
    wgtin.read_all("wgt",wgt);
    ss = new sourcescale(dtu, vel, wgt);
  } else {
    ss = new sourcescale(dtu, vel);
  }

  sregf srcout = sregf(srcin.get_axes());
  srcout.write_description("out");

  if(adj) {
    srcin.read_all("in", ssrc);
    if(ssrc->get_ndim() == 3) {
      ss->adjoint(false,psrc,ssrc);
    } else if(ssrc->get_ndim() == 4) {
      ss3.adjoint(false,psrc,ssrc);
    } else {
      seperr("Error: sourcescale operator only applies to 3D or 4D data.\n");
    }
    srcout.write_all("out",psrc);
  } else {
    srcin.read_all("in", psrc);
    if(psrc->get_ndim() == 3) {
      ss->forward(false,psrc,ssrc);
    } else if(psrc->get_ndim() == 4) {
      ss3.forward(false,psrc,ssrc);
    } else {
      seperr("Error: sourcescale operator only applies to 3D or 4D data.\n");
    }
    srcout.write_all("out",ssrc);
  }

  delete psrc; delete ssrc;
  delete vel;
}
