/**
 * A simple program to test
 * the 2D and 3D wavefield
 * interpolation operators
 * @author: Joseph Jennings
 * @version: 2017.01.07
 */

#include "sep_params.h"
#include "../main/wfldlint.h"
#include "../main/wfldlint3d.h"
#include "../main/simpleplot.h"

int main(int argc, char **argv) {

  sep_params pars = sep_params(argc, argv);

  int adj = pars.get_int("adj",0);
  /* Get output axis from command line*/
  int nts = 0; int ntu = 0;
  float dts = 0.0; float dtu = 0.0;
  float ots = 0.0; float otu = 0.0;
  if(adj) {
    nts = pars.get_int("nts",0);
    if(!nts)
      seperr("For adjoint, please enter the number of source wavelet samples (nts).\n");
    dts = pars.get_float("dts",0.0);
    if(dts == 0.0)
      seperr("Please enter the source wavelet sampling interval (dts).\n");
    ots = pars.get_float("ots",0.0);
  }
  else {
    ntu = pars.get_int("ntu",0);
    if(!ntu) seperr("Please enter the number of wavefield samples (ntu).\n");
    dtu = pars.get_float("dtu",0.0);
    if(dtu == 0.0) seperr("Please enter the wavefield sampling interval (dtu). \n");
    otu = pars.get_float("otu",0.0);
  }

  sregf srcin = sregf("in");
  hypercube_float *srcc = nullptr;
  hypercube_float *srcf = nullptr;
  wfldlint wi = wfldlint();
  if(adj) {
    /* Build sources */
    srcf = new hypercube_float(srcin.get_axes(), true);
    srcin.read_all("in",srcf);
    axis at = axis(nts,ots,dts);
    std::vector<axis> caxes;
    if(srcf->get_ndim() == 3) {
      caxes.push_back(srcin.get_axis(1)); caxes.push_back(srcin.get_axis(2));
      caxes.push_back(at);
      srcc = new hypercube_float(caxes, true);
      /* Apply adjoint operator and write result */
      wi.adjoint(false,srcc,srcf);
    } else if(srcf->get_ndim() == 4) {
      caxes.push_back(srcin.get_axis(1)); caxes.push_back(srcin.get_axis(2));
      caxes.push_back(srcin.get_axis(3)); caxes.push_back(at);
      srcc = new hypercube_float(caxes, true);
      wfldlint3d wi3 = wfldlint3d(srcc->get_axes(),srcf->get_axes());
      /* Apply adjoint operator and write result */
      wi3.adjoint(false,srcc,srcf);
    } else {
      seperr("Error: wfldlint-test only operates on 3D or 4D data.\n");
    }
    sregf out = sregf(caxes);
    out.write_description("out");
    out.write_all("out",srcc);
  } else {
    /* Build sources */
    srcc = new hypercube_float(srcin.get_axes(), true);
    srcin.read_all("in",srcc);
    axis at = axis(ntu,otu,dtu);
    std::vector<axis> faxes;
    if(srcc->get_ndim() == 3) {
      faxes.push_back(srcin.get_axis(1)); faxes.push_back(srcin.get_axis(2));
      faxes.push_back(at);
      srcf = new hypercube_float(faxes, true);
      /* Apply forward operator and write result */
      wi.forward(false,srcc,srcf);
    } else if(srcc->get_ndim() == 4) {
      faxes.push_back(srcin.get_axis(1)); faxes.push_back(srcin.get_axis(2));
      faxes.push_back(srcin.get_axis(3)); faxes.push_back(at);
      srcf = new hypercube_float(faxes, true);
      /* Apply forward operator and write result */
      wfldlint3d wi3 = wfldlint3d(srcc->get_axes(),srcf->get_axes());
      wi3.forward(false,srcc,srcf);
    } else {
      seperr("Error: wfldlint-test only operates on 3D or 4D data.\n");
    }
    sregf out = sregf(faxes);
    out.write_description("out");
    out.write_all("out",srcf);
  }
  delete srcc; delete srcf;
}
