/**
 * Tests the zero pad operator with a
 * simple small example
 * @author: Joseph Jennings
 * @version: 2017.01.07
 */
#include "sep_params.h"
#include "../main/zeropad.h"
#include "../main/zeropad3d.h"
#include "../main/simpleplot.h"

int main(int argc, char **argv) {

  /* Initialize IO */
  sep_params pars = sep_params(argc, argv);

  int adj = pars.get_int("adj",0);
  int nx  = pars.get_int("nx",100);
  int xs  = pars.get_int("xs",50 );
  int nz  = pars.get_int("nz",100);
  int zs  = pars.get_int("zs",0  );
  int ny  = pars.get_int("ny",0);
  int ys  = pars.get_int("ys",50);
  float dz = pars.get_float("dz",10.0);
  float dx = pars.get_float("dx",10.0);
  float dy = pars.get_float("dy",10.0);

  sregf srcin = sregf("in");
  hypercube_float *src, *psrc;

  zeropad zp    = zeropad(xs, zs);
  zeropad3d zp3 = zeropad3d(zs,xs,ys);
  if(adj) {
    /* Create source */
    psrc = new hypercube_float(srcin.get_axes(), true);
    std::vector<axis> saxes;
    if(psrc->get_ndim()== 3) {
      saxes.push_back(srcin.get_axis(3));
    } else if(psrc->get_ndim() == 4) {
      saxes.push_back(srcin.get_axis(4));
    } else {
      seperr("Error: zero pad operator only valid for 3D or 4D data.\n");
    }
    src = new hypercube_float(saxes, true);
    srcin.read_all("in",psrc);

    /* Apply operator */
    if(psrc->get_ndim() == 3) {
      zp.adjoint(false, src, psrc);
    } else if(psrc->get_ndim() == 4) {
      zp3.adjoint(false,src,psrc);
    }
    sregf out = sregf(src->get_axes());

    /* Write result */
    out.write_description("out");
    out.write_all("out", src);
    /* Remove the previous axes (for SEPlib) */
    if(psrc->get_ndim() == 3) {
      out.write_axis("out",2,axis(1,0.0,1.0));
      out.write_axis("out",3,axis(1,0.0,1.0));
    } else if(psrc->get_ndim() == 4){
      out.write_axis("out",2,axis(1,0.0,1.0));
      out.write_axis("out",3,axis(1,0.0,1.0));
      out.write_axis("out",4,axis(1,0.0,1.0));
    } else {
      seperr("Error: zero pad operator only valid for 3D or 4D data.\n");
    }
  }
  else {
    /* Create source */
    src = new hypercube_float(srcin.get_axes(), true);
    srcin.read_all("in",src);
    axis az = axis(nz, 0.0, dz);
    axis ax = axis(nx, 0.0, dx);
    axis ay = axis(ny, 0.0, dy);
    axis at = srcin.get_axis(1);
    std::vector<axis> paxes;
    if(!ny) {
      paxes.push_back(az); paxes.push_back(ax);
      paxes.push_back(at);
      psrc = new hypercube_float(paxes,true);
      /* Apply operator */
      zp.forward(false, src, psrc);
    }
    else {
      paxes.push_back(az); paxes.push_back(ax);
      paxes.push_back(ay); paxes.push_back(at);
      psrc = new hypercube_float(paxes,true);
      /* Apply operator */
      zp3.forward(false,src,psrc);
    }

    /* Write result */
    sregf out = sregf(psrc->get_axes());
    out.write_description("out");
    out.write_all("out", psrc);
  }

  delete psrc; delete src;
}
