/**
 * A program to test the ZeroPadimg
 * operator on a 2D image or 3D cube.
 * @author: Joseph Jennings
 * @version: 2017.01.07
 */

#include "sep_params.h"
#include "sregf.h"
#include "hypercube_float.h"
#include "../main/zeropadimg.h"
#include "../main/zeropadcub.h"
#include "../main/simpleplot.h"

int main(int argc, char **argv) {

  sep_params pars = sep_params(argc, argv);

  int adj    = pars.get_int("adj",0);
  int extx   = pars.get_int("extx",0);
  int extz   = pars.get_int("extz",0);
  int exty   = pars.get_int("exty",0);
  int xloc   = pars.get_int("xloc",0);
  int zloc   = pars.get_int("zloc",0);
  int yloc   = pars.get_int("yloc",0);
  float fill = pars.get_float("fill",0.0);

  sregf fin = sregf("in");
  hypercube_float *dat = nullptr;
  hypercube_float *pad = nullptr;
  zeropadimg zpi = zeropadimg(zloc,xloc,fill);
  zeropadcub zpc = zeropadcub(zloc,xloc,yloc,fill);
  if(adj) {
    /* Create pad */
    pad = new hypercube_float(fin.get_axes(), true);
    fin.read_all("in",pad);
    std::vector<axis> dax;
    if(pad->get_ndim() == 3) {
      /* Create dat */
      dax.push_back(axis(fin.get_axis(1).n - extz, fin.get_axis(1).o, fin.get_axis(1).d));
      dax.push_back(axis(fin.get_axis(2).n - extx, fin.get_axis(2).o, fin.get_axis(2).d));
      dax.push_back(axis(fin.get_axis(3).n - exty, fin.get_axis(3).o, fin.get_axis(3).d));
      dat = new hypercube_float(dax, true);
      zpc.adjoint(false,dat,pad);
    } else if(pad->get_ndim() == 2) {
      /* Create dat */
      dax.push_back(axis(fin.get_axis(1).n - extz, fin.get_axis(1).o, fin.get_axis(1).d));
      dax.push_back(axis(fin.get_axis(2).n - extx, fin.get_axis(2).o, fin.get_axis(2).d));
      dat = new hypercube_float(dax, true);
      zpi.adjoint(false,dat,pad);
    } else {
      seperr("Error: zeropadimg-test only operates on 2D and 3D data.\n");
    }
    sregf fout = sregf(dat->get_axes());
    fout.write_description("out");
    fout.write_all("out",dat);
  } else {
    /* Create dat */
    dat = new hypercube_float(fin.get_axes(), true);
    fin.read_all("in",dat);
    std::vector<axis> pax;
    if(dat->get_ndim() == 3) {
      pax.push_back(axis(fin.get_axis(1).n + extz, fin.get_axis(1).o, fin.get_axis(1).d));
      pax.push_back(axis(fin.get_axis(2).n + extx, fin.get_axis(2).o, fin.get_axis(2).d));
      pax.push_back(axis(fin.get_axis(3).n + extx, fin.get_axis(3).o, fin.get_axis(3).d));
      /* Create pad */
      pad = new hypercube_float(pax, true);
      zpc.forward(false,dat,pad);
    } else if(dat->get_ndim() == 2) {
      pax.push_back(axis(fin.get_axis(1).n + extz, fin.get_axis(1).o, fin.get_axis(1).d));
      pax.push_back(axis(fin.get_axis(2).n + extx, fin.get_axis(2).o, fin.get_axis(2).d));
      pad = new hypercube_float(pax, true);
      zpi.forward(false,dat,pad);
    } else {
      seperr("Error: zeropadimg-test only operates on 2D and 3D data.\n");
    }
    sregf fout = sregf(pad->get_axes());
    fout.write_description("out");
    fout.write_all("out",pad);
  }

  delete dat; delete pad;
}
