/**
 * A simple program to test the
 * Zero Pad wavefield operator
 * @author: Joseph Jennings
 * @version: 2016.12.13
 */

#include "sep_params.h"
#include "hypercube_float.h"
#include "../main/zeropadwfld.h"

int main(int argc, char **argv) {

  sep_params pars = sep_params(argc, argv);
  int adj    = pars.get_int("adj",0);
  int extx   = pars.get_int("extx",0);
  int extz   = pars.get_int("extz",0);
  int xloc   = pars.get_int("xloc",0);
  int zloc   = pars.get_int("zloc",0);
  float fill = pars.get_float("fill",0.0);

  sregf fin = sregf("in");
  hypercube_float *dat, *pad;
  zeropadwfld zpw = zeropadwfld(zloc,xloc,fill);
  if(adj) {
    /* Create pad */
    pad = new hypercube_float(fin.get_axes(), true);
    fin.read_all("in",pad);
    /* Create dat */
    std::vector<axis> dax;
    dax.push_back(axis(fin.get_axis(1).n - extz, fin.get_axis(1).o, fin.get_axis(1).d));
    dax.push_back(axis(fin.get_axis(2).n - extx, fin.get_axis(2).o, fin.get_axis(2).d));
    dax.push_back(pad->get_axis(3));
    dat = new hypercube_float(dax, true);
    zpw.adjoint(false,dat,pad);
    sregf fout = sregf(dat->get_axes());
    fout.write_description("out");
    fout.write_all("out",dat);
  } else {
    /* Create dat */
    dat = new hypercube_float(fin.get_axes(), true);
    fin.read_all("in",dat);
    /* Create pad */
    std::vector<axis> pax;
    pax.push_back(axis(fin.get_axis(1).n + extz, fin.get_axis(1).o, fin.get_axis(1).d));
    pax.push_back(axis(fin.get_axis(2).n + extx, fin.get_axis(2).o, fin.get_axis(2).d));
    pax.push_back(dat->get_axis(3));
    pad = new hypercube_float(pax, true);
    zpw.forward(false,dat,pad);
    sregf fout = sregf(pad->get_axes());
    fout.write_description("out");
    fout.write_all("out",pad);
  }

  delete dat; delete pad;
}
